package Workflow::App::CMS;

use strict;
use warnings;

use Exporter;
@Workflow::App::CMS::ISA = qw( Exporter );
use vars qw( @EXPORT_OK );
@EXPORT_OK = qw( rest_day is_limit );

use Encode;
use MT::AddressBook;
use CustomFields::Field;

use MT::Util qw( is_valid_email is_url dirify offset_time_list format_ts );
use MT::I18N qw( wrap_text );
use CustomFields::Util qw( get_meta save_meta );
use Workflow::CMS::Mail qw( workflow_build_mail );
use Workflow::Util qw( can_workflow use_revision get_lastest_revision get_lastest_revision_object );

use Data::Dumper;

sub pre_save {
    my ($eh, $app, $obj, $orig_obj) = @_;

    return 1 if !can_workflow($app);

    return 1 if $app->param('_autosave');

    return $app->error( $app->translate("expiration of a term.") )
        if is_limit();

    1;
}

sub post_save {
    my ($eh, $app, $obj, $orig_obj) = @_;

    return 1 if !use_revision($app);

    return 1 if $app->param('_autosave');

    # JÕXe[^XuJ(F҂)v̏ꍇ̂݁A30ɕύXiFҎgɂҏWJƌ邽߁j
    my $status = $obj->status;
    if ($status == 2) {

        my $revision = get_lastest_revision($app);
#        return 1 if !$revision;

#        $obj->current_revision($revision->rev_number) if $revision;

        my $rev = $obj->load_revision( { rev_number => $revision->rev_number } );
        if ( $rev && @$rev ) {
            $status = 30 if $rev->[0]->status == 20;
            $status = 30 if $rev->[0]->status > 2000;
        }
    }
    _send_mail($app, $obj, $status);
}

sub resend_mail {
    my $app = shift;

    return 1 if $app->param('_autosave');

    my $q     = $app->param;
    my $type  = $q->param('_type');

    my $revision = get_lastest_revision($app);
    return $app->error( "No Revision." ) if !$revision;

    my $entry = get_lastest_revision_object($app, $revision);
    return $app->error( $app->translate( "Invalid type [_1]", $type ) ) if $entry->class ne $type;

    require MT::Entry;
    require MT::Blog;
    require MT::Author;

    my $blog = MT::Blog->load( $entry->blog_id );
    my $author = $app->user;
    return $app->error(
        $app->translate( "No email address for user '[_1]'", $author->name ) )
      unless $author->email;

    my $cols = 72;
    my %params;
    $params{blog} = $blog;
    $params{entry} = $entry;
    $params{author} = $author;
    $params{message} = wrap_text( $q->param('resend_message'), $cols, '', '' );
    $params{approver_name} = $q->param('approver_name');

    if ( $entry->status > 2000 ) {
        $params{published} = 1;
    } else {
        $params{published} = 0;
    }

    my $entry_editurl = $app->uri(
        'mode' => 'view',
        args   => { 
            '_type' => $entry->class,
            blog_id => $entry->blog_id,
            id      => $entry->id,
        }
    ); 
    if ( $entry_editurl =~ m|^/| ) {
        my ($blog_domain) = $blog->archive_url =~ m|(.+://[^/]+)|;
        $entry_editurl = $blog_domain . $entry_editurl;
    }
    $params{entry_editurl} = $entry_editurl;

    my $entry_previewurl = $app->uri(
        'mode' => 'workflow_preview',
        args   => { 
            '_type' => $entry->class,
            blog_id => $entry->blog_id,
            id      => $entry->id,
        }
    ); 
    if ( $entry_previewurl =~ m|^/| ) {
        my ($blog_domain) = $blog->archive_url =~ m|(.+://[^/]+)|;
        $entry_previewurl = $blog_domain . $entry_previewurl;
    }
    $params{entry_previewurl} = $entry_previewurl;

    my @address_list;
    my $subject;
    my $body;

    $body = workflow_build_mail( $app, 'request-entry-approval', \%params );
    my $plugin = MT->component('Workflow');
    $subject =  $plugin->translate( "Request Approval [_2] - [_1]", $blog->name, $entry->title );

    for my $id ( $q->param('address') ) {
        if ($id =~ /^author/) {
            $id =~ s/author//;
            my $author = MT::Author->load({ id => $id });
            my $address = $author->name . ' <' . $author->email . '>';

            # [AhXdȂΏ
            push(@address_list, $address) unless grep { $_ eq $address } @address_list;
        }
    }

    return $app->error( $app->translate( "No email address" ) )
        if !@address_list;

    if ( $app->current_language ne 'ja' ) {
        $subject =~ s![\x80-\xFF]!!g;
    }

    my $charset = $app->config('MailEncoding') || $app->charset;

    require MT::Mail;
    foreach my $email (@address_list) {
        my %head = (
            id      => 'notify_entry',
            To      => $email,
            From    => $author->email,
            Subject => $subject,
        );
        $head{'Content-Type'} = qq(text/plain; charset="$charset");
        $head{To} = $email;
        MT::Mail->send( \%head, $body )
          or return $app->error(
            $app->translate("Error sending mail ([_1]); try another MailTransfer setting?",
                MT::Mail->errstr
            )
          );
        delete $head{To};
    }

    $app->add_return_arg( 'resend_mail' => 1 );
    $app->call_return;
}

sub request_preview_mail {
    my $app = shift;

    my $plugin = MT->component('Workflow');
    my $workflow_preview_mode = $plugin->get_config_value('workflow_preview_mode', 'blog:'.$app->blog->id);
    return $app->error(
        $plugin->translate( "No setting preview mode." ) )
      unless $workflow_preview_mode;;

    my $q     = $app->param;
    my $type  = $q->param('_type');

    my $revision = get_lastest_revision($app);
    return $app->error( "No Revision." ) if !$revision;

    my $entry = get_lastest_revision_object($app, $revision);
    return $app->error( $app->translate( "Invalid type [_1]", $type ) ) if $entry->class ne $type;

    require MT::Entry;
    require MT::Blog;
    require MT::Author;

    my $blog = MT::Blog->load( $entry->blog_id );
    my $author = $app->user;
    return $app->error(
        $app->translate( "No email address for user '[_1]'", $author->name ) )
      unless $author->email;

    my $cols = 72;
    my %params;
    $params{blog} = $blog;
    $params{entry} = $entry;
    $params{author} = $author;
    $params{message} = wrap_text( $q->param('preview_message'), $cols, '', '' );

    my @param = (
        'mode' => $workflow_preview_mode,
        args   => {
            '_type' => $entry->class,
            blog_id => $entry->blog_id,
            id      => $entry->id
        });
    my $entry_previewurl = $app->app_path . 'plugins/Workflow/mt-preview.cgi' . $app->uri_params(@param);

    if ( $entry_previewurl =~ m|^/| ) {
        my ($blog_domain) = $blog->archive_url =~ m|(.+://[^/]+)|;
        $entry_previewurl = $blog_domain . $entry_previewurl;
    }
    $params{entry_previewurl} = $entry_previewurl;

    my @address_list;
    my $subject;
    my $body;

    $body = workflow_build_mail( $app, 'request-preview', \%params );
    $subject =  $plugin->translate( "Request Preview [_2] - [_1]", $blog->name, $entry->title );

    for my $id ( $q->param('address') ) {
        if ($id =~ /^address/) {
            $id =~ s/address//;
            my $addressbook = MT::AddressBook->load({ id => $id });
            my $address = $addressbook->name . ' <' . $addressbook->email . '>';
            push(@address_list, $address);
        }
    }
    push(@address_list, $q->param('else_email')) if ( $q->param('else_email') );

    return $app->error( $app->translate( "No email address" ) )
        if !@address_list;

    if ( $app->current_language ne 'ja' ) {
        $subject =~ s![\x80-\xFF]!!g;
    }

    my $charset = $app->config('MailEncoding') || $app->charset;

    require MT::Mail;
    foreach my $email (@address_list) {
        my %head = (
            id      => 'notify_entry',
            To      => $email,
            From    => $author->email,
            Subject => $subject,
        );
        $head{'Content-Type'} = qq(text/plain; charset="$charset");
        $head{To} = $email;
        MT::Mail->send( \%head, $body )
          or return $app->error(
            $app->translate("Error sending mail ([_1]); try another MailTransfer setting?",
                MT::Mail->errstr
            )
          );
        delete $head{To};
    }

    $app->add_return_arg( 'send_preview' => 1 );
    $app->call_return;
}

sub save_snapshot_wf {
    my $app = shift;

    return if !use_revision($app);

    return 1 if $app->param('_autosave');

    return $app->error( $app->translate("expiration of a term.") )
        if is_limit();

    my $q     = $app->param;
    my $type  = $q->param('_type');
    my $id    = $q->param('id');
    my $blog_id    = $q->param('blog_id');
    my $param = {};

    return $app->errtrans("Invalid request.")
        unless $type;

    $app->validate_magic() or return;

    my $class = $app->model($type)
        or return $app->errtrans( "Invalid type [_1]", $type );

    return $app->error( $app->translate( "No ID" ) ) if !$id;

    my $revision = get_lastest_revision($app);
    return $app->error( "No Revision." ) if !$revision;

    my $obj = get_lastest_revision_object($app, $revision);
    return $app->error( $app->translate( "Invalid type [_1]", $type ) ) if $obj->class ne $type;

    my $original = $obj->clone();
    $original->{__tags} = $obj->{__tags};

    my @category_list;
    my $categories = $obj->cache_property( 'categories' );
    foreach my $c (@{$categories}) {
        push @category_list, $c->id;
    }
    $original->{__cats} = \@category_list;

    my $names    = $obj->column_names;
    my %values   = map { $_ => ( scalar $q->param($_) ) } @$names;

    if ( ( 'entry' eq $type ) || ( 'page' eq $type ) ) {
        $app->_translate_naughty_words($obj);
    }
    delete $values{'id'} if exists( $values{'id'} ) && !$values{'id'};

    $obj->set_values( \%values );

    my $ao_d = $q->param('authored_on_date');
    my $ao_t = $q->param('authored_on_time');
##################### MT6
    my $uo_d = $app->param('unpublished_on_date');
    my $uo_t = $app->param('unpublished_on_time');
##################### MT6
    if ( $ao_d ) {
        my %param = ();
        my $ao    = $ao_d . ' ' . $ao_t;
        unless ( $ao
            =~ m!^(\d{4})-(\d{1,2})-(\d{1,2})\s+(\d{1,2}):(\d{1,2})(?::(\d{1,2}))?$!
            )
        {
            $param{error} = $app->translate(
                "Invalid date '[_1]'; 'Published on' dates must be in the format YYYY-MM-DD HH:MM:SS.",
                $ao
            );
        }
        unless ( $param{error} ) {
            my $s = $6 || 0;
            $param{error} = $app->translate(
                "Invalid date '[_1]'; 'Published on' dates should be real dates.",
                $ao
                )
                if (
                   $s > 59
                || $s < 0
                || $5 > 59
                || $5 < 0
                || $4 > 23
                || $4 < 0
                || $2 > 12
                || $2 < 1
                || $3 < 1
                || ( MT::Util::days_in( $2, $1 ) < $3
                    && !MT::Util::leap_day( $0, $1, $2 ) )
                );
        }
        $param{return_args} = $app->param('return_args');
        return $app->forward( "view", \%param ) if $param{error};
        my $ts = sprintf "%04d%02d%02d%02d%02d%02d", $1, $2, $3, $4, $5, ( $6 || 0 );
        $obj->authored_on($ts);
    }
##################### MT6
    if ( $uo_d || $uo_t ) {
        my %param = ();
        my $uo    = $uo_d . ' ' . $uo_t;
        $param{error} = $app->translate(
            "Invalid date '[_1]'; 'Unpublished on' dates must be in the format YYYY-MM-DD HH:MM:SS.",
            $uo
            )
            unless ( $uo
            =~ m!^(\d{4})-(\d{1,2})-(\d{1,2})\s+(\d{1,2}):(\d{1,2})(?::(\d{1,2}))?$!
            );
        unless ( $param{error} ) {
            my $s = $6 || 0;
            $param{error} = $app->translate(
                "Invalid date '[_1]'; 'Unpublished on' dates should be real dates.",
                $uo
                )
                if (
                   $s > 59
                || $s < 0
                || $5 > 59
                || $5 < 0
                || $4 > 23
                || $4 < 0
                || $2 > 12
                || $2 < 1
                || $3 < 1
                || ( MT::Util::days_in( $2, $1 ) < $3
                    && !MT::Util::leap_day( $0, $1, $2 ) )
                );
        }
        my $ts = sprintf "%04d%02d%02d%02d%02d%02d", $1, $2, $3, $4, $5,
            ( $6 || 0 );
        require MT::DateTime;
        unless ( $param{error} ) {
            $param{error} = $app->translate(
                "Invalid date '[_1]'; 'Unpublished on' dates should be dates in the future.",
                $uo
                )
                if (
                MT::DateTime->compare(
                    blog => $app->blog,
                    a    => { value => time(), type => 'epoch' },
                    b    => $ts
                ) > 0
                );
        }
        if ( !$param{error} && $obj->authored_on ) {
            $param{error} = $app->translate(
                "Invalid date '[_1]'; 'Unpublished on' dates should be later than the corresponding 'Published on' date.",
                $uo
                )
                if (
                MT::DateTime->compare(
                    blog => $app->blog,
                    a    => $obj->authored_on,
                    b    => $ts
                ) > 0
                );
        }
        $param{show_input_unpublished_on} = 1 if $param{error};
        $param{return_args} = $app->param('return_args');
        return $app->forward( "view", \%param ) if $param{error};
        $obj->unpublished_on($ts);
    }
    else {
        $obj->unpublished_on(undef);
    }
##################### MT6

    $obj->current_revision($revision->rev_number);

    $obj->status(10) if $obj->status == 40;

    require MT::Asset;
    require MT::ObjectAsset;
    my $include_asset_ids = $app->param('include_asset_ids');
    my @asset_ids         = split( ',', $include_asset_ids );
    my $obj_assets        = ();
    my @obj_assets        = MT::ObjectAsset->load(
        { object_ds => 'entry', object_id => $obj->id } );
    foreach my $obj_asset (@obj_assets) {
        my $asset_id = $obj_asset->asset_id;
        $obj_assets->{$asset_id} = 1;
    }
    my $seen = ();
    foreach my $asset_id (@asset_ids) {
        my $obj_asset = MT::ObjectAsset->load(
            {   asset_id  => $asset_id,
                object_ds => 'entry',
                object_id => $obj->id
            }
        );
        unless ($obj_asset) {
            my $obj_asset = new MT::ObjectAsset;
            $obj_asset->blog_id($blog_id);
            $obj_asset->asset_id($asset_id);
            $obj_asset->object_ds('entry');
            $obj_asset->object_id( $obj->id );
            $obj_asset->save;
        }
        $seen->{$asset_id} = 1;
    }
    foreach my $asset_id ( keys %{$obj_assets} ) {
        unless ( $seen->{$asset_id} ) {
            my $obj_asset = MT::ObjectAsset->load(
                {   asset_id  => $asset_id,
                    object_ds => 'entry',
                    object_id => $obj->id
                }
            );
            $obj_asset->remove;
        }
    }

    my $cat;
    my $cat_ids = $q->param('category_ids');
    if ($cat_ids) {
        my @cats = split /,/, $cat_ids;
        if (@cats) {
            my $primary_cat = $cats[0];
            $cat = MT::Category->load(
                { id => $primary_cat, blog_id => $blog_id } );
            my @categories;
            @categories = MT::Category->load( { id => \@cats, blog_id => $blog_id } ) if @cats;
            $obj->cache_property( 'category',   undef, $cat );
            $obj->cache_property( 'categories', undef, \@categories );
        }
    }
    else {
        $obj->cache_property( 'category', undef, undef );
        $obj->cache_property( 'categories', undef, [] );
    }

    if ($q->param('tags')) {
        my @tag_names = split(/\s*,\s*/,$q->param('tags'));
        my @save_tags;
        foreach my $tag_name (@tag_names) {
            my $tag;
            my @tags = MT::Tag->load({name => $tag_name});
            if (!@tags) {
                $tag = MT::Tag->new;
                $tag->name($tag_name);
                $tag->save;
            } else {
               $tag = shift @tags;
            }
            push(@save_tags, $tag);
        }
        $obj->{__tag_objects} = \@save_tags;
    } else {
        delete $obj->{__tags};
        delete $obj->{__tag_objects};
    }

    _set_customfield_objs($app, $obj);

    _gather_changed_cols( $obj, $original, $app );

    if ($obj->{changed_revisioned_cols}) {
        my $col = 'max_revisions_' . $obj->datasource;
        if ( my $blog = $obj->blog ) {
            my $max = $blog->$col;
            $obj->handle_max_revisions($max);
        }

        my $revision = $obj->save_revision( $q->param('revision-note') );
        $app->add_return_arg( 'saved_snapshot' => 1 );
    }
    else {
        $app->add_return_arg( 'no_snapshot' => 1 );
    }

    my $status = $q->param('status');

    # fix bug
    if (($status == 10) || ($status == 20) || ($status == 40) || ($status > 2000)) {
        my $entry = MT::Entry->load($id);
        my $revision_number = $obj->current_revision() + 1;
        $entry->current_revision( $revision_number );
        $entry->save;
    }

    if ($status == 20 || $status == 40 || $status > 2000) {
        _send_mail($app, $obj, $status);
    }

    $app->add_return_arg( 'id' => $obj->id ) if !$original->id;
    $app->call_return;
}

sub _gather_changed_cols {
    my ( $obj, $orig_obj, $app ) = @_;

    my @changed_cols;
    my $revisioned_cols = $obj->revisioned_columns;

    my %date_cols = map { $_ => 1 }
        @{ $obj->columns_of_type( 'datetime', 'timestamp' ) };

    foreach my $col (@$revisioned_cols) {
        next if $orig_obj && $obj->$col eq $orig_obj->$col;
        next
            if $orig_obj
                && exists $date_cols{$col}
                && $orig_obj->$col eq MT::Object::_db2ts( $obj->$col );

        push @changed_cols, $col;
    }

#    $obj->{changed_revisioned_cols} = \@changed_cols
#        if @changed_cols;

#    return 1 if @changed_cols;

    my $tag_changed = 0;

    my @objecttags = @{ $orig_obj->{__tags} };

    my @tag_names = split /\s*,\s*/,
            ( $app->param('tags') || '' );

    $tag_changed = 1 if scalar(@tag_names) != scalar(@objecttags);

    unless ($tag_changed) {
        if (@tag_names && @objecttags) {
            my @sort_tag_names = sort { $a <=> $b } @tag_names;
            my @sort_objecttags = sort { $a <=> $b } @objecttags;
            $tag_changed = 1 if join('', @sort_tag_names) ne join('', @sort_objecttags);
        }
    }
    push @changed_cols, 'tags' if $tag_changed;

    my $cat_changed = 0;

    my @category_list = @{ $orig_obj->{__cats} };

    my @category_ids;
    eval {
        @category_ids = split /\s*,\s*/,
            ( $app->param('category_ids') || '' );
        @category_ids = grep { $_ != -1 } @category_ids;
    };

    $cat_changed = 1 if scalar(@category_ids) != scalar(@category_list);

    unless ($cat_changed) {
        if (@category_ids && @category_list) {
            my @sort_category_ids = sort { $a <=> $b } @category_ids;
            my @sort_category_list = sort { $a <=> $b } @category_list;
            $cat_changed = 1 if join('', @sort_category_ids) ne join('', @sort_category_list);
        }
    }

    push @changed_cols, 'categories' if $cat_changed;

#    object asset
#    my $orig_object_assets = $obj->cache_property( 'orig_object_assets' );
#    my $object_assets = $obj->cache_property( 'object_assets' );
#    return 1 if !$orig_object_assets && !$object_assets;

#    my @orig_assets = sort { $a <=> $b } @$orig_object_assets;
#    my @assets = sort { $a <=> $b } @$object_assets;

#    $obj->cache_property( 'orig_object_assets', undef, undef );
#    return 1 if join('', @orig_assets) == join('', @assets);

#    my $changed_cols = $obj->{changed_revisioned_cols};
#    push @$changed_cols, 'assets';

#    return 1 if @changed_cols;

    my $meta = get_meta($orig_obj);
    while ( my ( $key, $val ) = each %$meta ) {
        if ( my $newval = $app->param( 'customfield_' . $key ) ) {
            if ( $val ne $newval ) {
                push @changed_cols, "field.$key";
            }
        } else {
            my $field = CustomFields::Field->load(
                {   $app->blog->id
                    ? ( blog_id => [ $app->blog->id, 0 ] )
                    : ( blog_id => $app->blog->id ),
                    basename => $key
                });
            if ( $field ) {
                if ( $field->type eq 'datetime' ) {
                    if ( $field->options eq 'datetime' ) {
                        $newval = $app->param( 'd_customfield_' . $key ) . " " .
                                  $app->param( 't_customfield_' . $key );
                    } elsif ( $field->options eq 'date' ) {
                        $newval = $app->param( 'd_customfield_' . $key );
                        $val =~ s/(\d{4}-\d{2}-\d{2}) \d{2}:\d{2}:\d{2}/$1/;
                    } elsif ( $field->options eq 'time' ) {
                        $newval = $app->param( 't_customfield_' . $key );
                        $val =~ s/\d{4}-\d{2}-\d{2} (\d{2}:\d{2}:\d{2})/$1/;
                    }
                    if ( $val ne $newval ) {
                        push @changed_cols, "field.$key";
                    }
                } else {
                    push @changed_cols, "field.$key" if $val;
                }
            }
        }
    }

    $obj->{changed_revisioned_cols} = \@changed_cols
        if @changed_cols;

    1;
}

sub set_changed_cols {
    my ( $cb, $obj, $orig_obj, $app ) = @_;

    return 1 if !$obj;
    return 1 if !$orig_obj;

    # LWorkflowC^[̂̂łΎs
    my $author = MT::Author->load($obj->author_id);
    my $blog_id = MT::Author->load($obj->blog_id);

    require MT::Association;
    require MT::Role;
    my ( $join_terms, $join_args );
    $join_terms->{ author_id } = $author->id;
    $join_args->{ blog_id } = $blog_id;
    $join_args->{ unique } = 1;
    my $args;
    $args->{ join } = MT::Association->join_on( 'role_id', $join_terms, $join_args );
    my $iter = MT::Role->load_iter( undef, $args );

    my $plugin = MT->component('Workflow');
    my $role_name = $plugin->translate('writer');

    my $flag = 0;
    while (my $role = $iter->()) {
        $flag = 1 if $role->name eq $role_name;
    }
    return if !$flag;
    if ( $obj->status == $orig_obj->status ) {
        my $changed_cols = $obj->{changed_revisioned_cols};
        push @$changed_cols, 'status';
        $obj->{changed_revisioned_cols} = $changed_cols;
    }
    1;
}

sub _set_customfield_objs {
    my ( $app, $obj ) = @_;
    return 1 unless $app->isa('MT::App');

    $app->run_callbacks( 'api_save_filter.entry', $app );

    my $q = $app->param;
    my $blog_id = $app->param('blog_id') || 0;

    return 1 if !$q->param('customfield_beacon');
    require CustomFields::Field;

    my $meta = {};
    my %date_fields;
    foreach ( $q->param() ) {
        next if $_ eq 'customfield_beacon';
        if (m/^customfield_(.*?)$/) {
            my $field_name = $1;
            if (m/^customfield_(.+?)_cb_beacon$/) {
                $field_name = $1;
                $meta->{$field_name}
                    = defined( $q->param("customfield_$field_name") )
                    ? $q->param("customfield_$field_name")
                    : '0';
            }
            else {
                my $field = CustomFields::Field->load(
                    {   $blog_id
                        ? ( blog_id => [ $blog_id, 0 ] )
                        : ( blog_id => $blog_id ),
                        basename => $field_name
                    }
                ) or next;
                $meta->{$field_name} = $q->param("customfield_$field_name");
            }
        } elsif (m/^([td]_)customfield_(.*?)$/) {
            my ( $td, $field_name ) = ( $1, $2 );
            if ( $td ) {
                $date_fields{$field_name} = 1;
            }
        }

    }
    for my $field_name ( keys %date_fields ) {
        my $field = CustomFields::Field->load(
            {   $blog_id
                ? ( blog_id => [ $blog_id, 0 ] )
                : ( blog_id => $blog_id ),
                basename => $field_name
            }
        ) or next;
        if ( $field->options eq 'datetime' ) {
            $meta->{$field_name} = $q->param("d_customfield_$field_name") . " " .
                                   $q->param("t_customfield_$field_name");
        }
        elsif ( $field->options eq 'date' ) {
            if ( $q->param("d_customfield_$field_name")) {
                $meta->{$field_name} = $q->param("d_customfield_$field_name") . ' 00:00:00';
            } else {
                $meta->{$field_name} = '';
            }
        }
        elsif ( $field->options eq 'time' ) {
            if ( $q->param("t_customfield_$field_name")) {
                $meta->{$field_name} = '1970-01-01 ' . $q->param("t_customfield_$field_name");
            } else {
                $meta->{$field_name} = '';
            }
        }
    }
    _save_meta_wf( $obj, $meta ) if %$meta;

    1;
}

sub _save_meta_wf {
    my $plugin = MT->component("Commercial");
    my ( $obj, $meta ) = @_;
    return 0 unless $obj;
    my $obj_type
        = $obj->can('class_type') ? $obj->class_type : $obj->datasource;

    if ( $obj->has_meta ) {
        foreach my $mf ( keys %$meta ) {
            if ( $obj->has_meta( 'field.' . $mf ) ) {
                $obj->meta( 'field.' . $mf, $meta->{$mf} );
            }
        }
    }
    else {
        my $id = $obj->id;

        require MT::PluginData;
        my $meta_data = MT::PluginData->get_by_key(
            { plugin => 'CustomFields', key => "${obj_type}_${id}" } );
        $meta_data->data( { customfields => $meta } );
    }
    sync_assets($obj) unless $obj->isa('MT::Entry');

    return 1;
}

sub _send_mail {
    my ($app, $obj, $old_status) = @_;

    my $entry_id = $obj->id;
    my $q        = $app->param;
    my $reviewee = $app->can_do('request_entry_approval');
    my $reviewer = $app->can_do('approve_entry');

    return if $reviewee && $obj->status == 1;

    require MT::Entry;
    require MT::Blog;
    require MT::Author;

    my $entry = MT::Entry->load($entry_id)
      or return $app->error(
        $app->translate( "No such entry '[_1]'", $entry_id ) );

    if ($old_status == 20 || $old_status > 2000) {
        for my $col (qw( title text text_more excerpt keywords convert_breaks
                     allow_comments allow_pings basename
                     authored_on modified_on status )) {
            $entry->$col($obj->$col);
        }
    }

    my $blog = MT::Blog->load( $entry->blog_id );
    my $author;
    if ($reviewee) {
        $author = $entry->author;
    } elsif ($reviewer) {
        $author = $app->user;
    }
    return $app->error(
        $app->translate( "No email address for user '[_1]'", $author->name ) )
      unless $author->email;

    my $cols = 72;
    my %params;
    $params{blog} = $blog;
    $params{entry} = $entry;
    $params{author} = $author;

    # p[^message͏F/߂̂
#    $params{message} = wrap_text( $q->param('message'), $cols, '', '' );

    my @address_list;
    my $subject;
    my $body;

    my $plugin = MT->component('Workflow');

    if ($reviewee) {
        my $cfg = $app->config;
        my $cgi_path = $app->config('CGIPath');

        my $entry_editurl =
            $cgi_path 
            . $cfg->AdminScript
            . $app->uri_params(
                  'mode' => 'view',
                  args   => {
                      '_type' => $obj->class,
                      blog_id => $entry->blog_id,
                      id      => $entry->id,
                  },
            );
        if ( $entry_editurl =~ m|^/| ) {
            my ($blog_domain) = $blog->archive_url =~ m|(.+://[^/]+)|;
            $entry_editurl = $blog_domain . $entry_editurl;
        }
        $params{entry_editurl} = $entry_editurl;

        my $entry_previewurl =
            $cgi_path 
            . $cfg->AdminScript
            . $app->uri_params(
                  'mode' => 'workflow_preview',
                  args   => {
                      '_type' => $obj->class,
                      blog_id => $entry->blog_id,
                      id      => $entry->id,
                  },
            );
        if ( $entry_previewurl =~ m|^/| ) {
            my ($blog_domain) = $blog->archive_url =~ m|(.+://[^/]+)|;
            $entry_previewurl = $blog_domain . $entry_previewurl;
        }
        $params{entry_previewurl} = $entry_previewurl;

        if ($obj->status == 3 || $obj->status == 1001 ) {
            $params{published} = 0;
        } else {
            $params{published} = 1;
        }

        # F˗bZ[W
        $params{message} = wrap_text( $q->param('request_approve_message'), $cols, '', '' );

        $params{review} = 'reviewee';

        # J̓rWf[^𗘗p
        if ( $params{published} ) {
            my $type  = $q->param('_type');
            my $revision = get_lastest_revision($app);

            # VKLŏF˗̂ŃXLbv
#           return $app->error( "No Revision." ) if !$revision;

            $entry = get_lastest_revision_object($app, $revision);
            return $app->error( $app->translate( "Invalid type [_1]", $type ) ) if $entry->class ne $type;

            $params{entry} = $entry if $revision;
        }

        $body = workflow_build_mail( $app, 'request-entry-approval', \%params );
        $subject =  $plugin->translate( "Request Approval [_2] - [_1]", $blog->name, $entry->title );
        for my $id ( $q->param('address') ) {
            if ($id =~ /^author/) {
                $id =~ s/author//;
                my $author = MT::Author->load({ id => $id });
                my $address = $author->name . ' <' . $author->email . '>';
                push(@address_list, $address);
            }
        }
    } elsif($reviewer) {

        my $cfg = $app->config;
        my $cgi_path = $app->config('CGIPath');

        my $entry_editurl =
            $cgi_path 
            . $cfg->AdminScript
            . $app->uri_params(
                  'mode' => 'view',
                  args   => {
                      '_type' => $obj->class,
                      blog_id => $entry->blog_id,
                      id      => $entry->id,
                  },
            );
        if ( $entry_editurl =~ m|^/| ) {
            my ($blog_domain) = $blog->archive_url =~ m|(.+://[^/]+)|;
            $entry_editurl = $blog_domain . $entry_editurl;
        }
        $params{entry_editurl} = $entry_editurl;

        my $entry_previewurl =
            $cgi_path 
            . $cfg->AdminScript
            . $app->uri_params(
                  'mode' => 'workflow_preview',
                  args   => {
                      '_type' => $obj->class,
                      blog_id => $entry->blog_id,
                      id      => $entry->id,
                  },
            );
        if ( $entry_previewurl =~ m|^/| ) {
            my ($blog_domain) = $blog->archive_url =~ m|(.+://[^/]+)|;
            $entry_previewurl = $blog_domain . $entry_previewurl;
        }
        $params{entry_previewurl} = $entry_previewurl;

        $params{review} = 'reviewer';
        $params{approver_name} = $q->param('approver_name');

        my $entry = MT::Entry->load($q->param('id'));
        my $writer = MT::Author->load($entry->author_id);

        if (($old_status == 40) || $obj->status == 1) {
            $params{published} = 1 if $old_status == 40;

            # J̓rWf[^𗘗p
            if ( $old_status == 40 ) {
                my $type  = $q->param('_type');
                my $revision = get_lastest_revision($app);
                return $app->error( "No Revision." ) if !$revision;

                $entry = get_lastest_revision_object($app, $revision);
                return $app->error( $app->translate( "Invalid type [_1]", $type ) ) if $entry->class ne $type;

                $params{entry} = $entry;
            }

            # ߂bZ[W
            $params{message} = wrap_text( $q->param('message'), $cols, '', '' );

            $body = workflow_build_mail( $app, 'no-approve-entry', \%params );
            $subject =  $plugin->translate( "No Approve [_2] - [_1]", $blog->name, $entry->title );

            push(@address_list, $writer->email);

        } elsif ($old_status == 30 || $obj->status == 2) {
            $params{published} = 1 if $old_status == 30;

            # FbZ[W
            $params{message} = wrap_text( $q->param('message'), $cols, '', '' );

            $body = workflow_build_mail( $app, 'approve-entry', \%params );
            $subject =  $plugin->translate( "Approve [_2] - [_1]", $blog->name, $entry->title );

            push(@address_list, $writer->email);

        } else {
            $params{published} = 1 if $old_status > 2000;

            # F˗bZ[W
            $params{message} = wrap_text( $q->param('request_approve_message'), $cols, '', '' );

            # J̓rWf[^𗘗p
            if ( $old_status > 2000 ) {
                my $type  = $q->param('_type');
                my $revision = get_lastest_revision($app);
                return $app->error( "No Revision." ) if !$revision;

                $entry = get_lastest_revision_object($app, $revision);
                return $app->error( $app->translate( "Invalid type [_1]", $type ) ) if $entry->class ne $type;

                $params{entry} = $entry;
            }

            $body = workflow_build_mail( $app, 'request-entry-approval', \%params );
            $subject =  $plugin->translate( "Request Approval [_2] - [_1]", $blog->name, $entry->title );

            for my $id ( $q->param('address') ) {
                if ($id =~ /^author/) {
                    $id =~ s/author//;
                    my $author = MT::Author->load({ id => $id });
                    my $address = $author->name . ' <' . $author->email . '>';
                    push(@address_list, $address);
                }
            }
        }
    }

    if ( $app->current_language ne 'ja' ) {
        $subject =~ s![\x80-\xFF]!!g;
    }

    my $charset = $app->config('MailEncoding') || $app->charset;

    require MT::Mail;
    foreach my $email (@address_list) {
        my %head = (
            id      => 'notify_entry',
            To      => $email,
            From    => $app->user->email,
            Subject => $subject,
        );
        $head{'Content-Type'} = qq(text/plain; charset="$charset");
        $head{To} = $email;
        MT::Mail->send( \%head, $body )
          or return $app->error(
            $app->translate("Error sending mail ([_1]); try another MailTransfer setting?",
                MT::Mail->errstr
            )
          );
        delete $head{To};
    }
}

sub pack_revision_asset {
    my $obj = shift;
    my ($values) = @_;

    $values->{__rev_assets} = $obj->cache_property( 'object_assets' );
    $values;
}

sub unpack_revision_asset {
    my $obj = shift;
    my ($packed_obj) = @_;

    if ( my $rev_assets = delete $packed_obj->{__rev_assets} ) {
        $obj->clear_cache('assets');
        $obj->cache_property( 'object_assets', undef, \@$rev_assets );
    }
}

sub init_request {
    my $plugin = MT->component("Workflow");
    my $workflow_column_multistage = $plugin->get_config_value('workflow_column_multistage', 'system') || '';
    if ($workflow_column_multistage eq ',') {
        $plugin->set_config_value('workflow_column_multistage', time(), 'system');
    }
}

sub is_limit {
    return (rest_day() > 0) ? 0 : 1;
}

sub rest_day {
    my $plugin = MT->component("Workflow");
    my $workflow_column_multistage = $plugin->get_config_value('workflow_column_multistage', 'system') || '';
    my $duration = time() - $workflow_column_multistage;
    my $rest_day = 31 - int($duration / (24 * 3600));
    return ($rest_day > 0) ? $rest_day : 0;
}

1;
