package Workflow::CMS::Entry;

use strict;
use warnings;

use Encode;
use MT::AddressBook;
use MT::Approver;
use CustomFields::App::CMS;

use MT::Util qw( format_ts relative_date );
use CustomFields::Util qw( get_meta _get_html );
use Workflow::Util qw( can_workflow use_revision get_lastest_revision get_lastest_revision_object get_authors_with_order );
use Workflow::App::Approver qw( get_max_order );
use Workflow::App::CMS qw( rest_day is_limit );

use Data::Dumper;

my $approver_prefix = '123';

sub cms_pre_preview {
    my ($cb, $app, $obj, $data) = @_;

    my $revision = get_lastest_revision($app);
    return 1 if !$revision;

    $obj->current_revision($revision->rev_number) if $revision;

    my $rev = $obj->load_revision( { rev_number => $revision->rev_number } );
    push @$data,
        {
            data_name => 'status',
            data_value => $rev->[0]->status
    };
}

sub delete_action {
    my ($cb, $app, $param, $tmpl) = @_;

    return if !can_workflow($app);

    $param->{use_actions} = 0;
}

sub preview_entry {
    my ($cb, $app, $template) = @_;

    return if !can_workflow($app);

    my $old;
    my $new;

    $old = <<HTML;
  <button
     mt:mode="save_entry"
     type="submit"
     name="save"
     value="save"
     accesskey="s"
     title="<\$mt:var name="save_button_title"\$>"
     class="save action primary button">
    <mt:var name="save_button_value">
  </button>
HTML
    $old = quotemeta($old);

    $new = <<HTML;
HTML

    $$template =~ s/$old//;

}

sub list_revision {
    my ($cb, $app, $terms, $args, $param, $hasher) = @_;

    return if !can_workflow($app);

    my $q = $app->param;
    my $r = $q->param('r');

    my $id = $terms->{entry_id};
    my $obj = MT::Entry->load($id);
    my $type = 'entry';
    my $blog = $app->blog;
    my $lang  = $app->user ? $app->user->preferred_language : undef;
    my %users;
    my $js = $param->{rev_js};
    unless ($js) {
        $js
            = "location.href='"
            . $app->uri
            . '?__mode=view&amp;_type='
            . $app->param('_type');
        if ( my $id = $obj->id ) {
            $js .= '&amp;id=' . $id;
        }
        if ( defined $blog ) {
            $js .= '&amp;blog_id=' . $blog->id;
        }
    }
    $$hasher = sub {
        my ( $rev, $row ) = @_;

        if ( my $ts = $rev->created_on ) {
            $row->{created_on_formatted}
                = format_ts( MT::App::CMS::LISTING_DATE_FORMAT(),
                $ts, $blog, $lang );
            $row->{created_on_time_formatted}
                = format_ts( MT::App::CMS::LISTING_TIMESTAMP_FORMAT(),
                $ts, $blog, $lang );
            $row->{created_on_relative} = relative_date( $ts, time, $blog );
        }
        if ( $row->{created_by} ) {
            my $created_user = $users{ $row->{created_by} }
                ||= MT::Author->load( $row->{created_by} );
            if ($created_user) {
                $row->{created_by} = $created_user->nickname;
            }
            else {
                $row->{created_by} = $app->translate('(user deleted)');
            }
        }
        my $revision    = $obj->object_from_revision($rev);
        my $column_defs = $obj->column_defs;

        if ( ( 'entry' eq $type ) || ( 'page' eq $type ) ) {
            $row->{rev_status} = $revision->[0]->status;
        }
        $row->{rev_js}  = $js . '&amp;r=' . $row->{rev_number} . "'";

        my ($terms, $args);
        $terms->{'entry_id'} = $revision->[0]->id;
        $args->{sort} = 'rev_number';
        $args->{direction} = 'descend';
        $args->{limit} = 1;
        my $rev_model = MT->instance->model($type.':revision');
        my @revs = $rev_model->load($terms, $args);
        my $rev = shift @revs || undef;

        if ($r) {
            $row->{is_current} = $r == $row->{rev_number};
        } else {
            $row->{is_current} = $rev->rev_number == $row->{rev_number};
        }
    };

}

sub set_entry_param {
    my ($cb, $app, $param, $tmpl) = @_;
#    my $author = $app->user;

    return 1 if $app->param('_autosave');
    return 1 if !$app->param('id');

    my $reviewee = $app->can_do('request_entry_approval');
    my $reviewer = $app->can_do('approve_entry');

    my $q = $app->param;
    my $type = $q->param('_type');

    my $id = $q->param('id');
    my $model = $app->model($type);
    my $rev_model = $app->model($type.':revision');
    return 1 unless $rev_model;

    my $obj;
    my $before_status;
    my ($terms, $args);
    $terms->{entry_id} = $id;
    $args->{sort} = 'rev_number';
    $args->{direction} = 'descend';

    ### preview bug fix
    if (!$q->param('reedit') && !$param->{error}) {
    ###
    my $use_revision = use_revision($app);
    if ($use_revision) {
        $args->{limit} = 1;
        my @revs = $rev_model->load($terms, $args);

        my $revision = shift @revs || undef;
        if ($revision) {
            my $revision_number = $q->param('r') ? $q->param('r') : $revision->rev_number;

            my @objs = $model->load($id);
            $obj = shift @objs || undef;
            my $rev = $obj->load_revision( { rev_number => $revision_number } );
            if ( $rev && @$rev ) {
                $obj = $rev->[0];

                $param->{entry} = $obj;

                my $values = $obj->get_values;
                $param->{$_} = $values->{$_} foreach keys %$values;
                $param->{rev_date} = format_ts( "%Y-%m-%d %H:%M:%S",
                    $obj->modified_on, $app->blog,
                    $app->user ? $app->user->preferred_language : undef );

                $param->{'authored_on_date'} = format_ts( "%Y-%m-%d", $obj->authored_on, $app->blog,
                    $app->user ? $app->user->preferred_language : undef );
                $param->{'authored_on_time'} = format_ts( "%H:%M:%S", $obj->authored_on, $app->blog,
                    $app->user ? $app->user->preferred_language : undef );
##################### MT6
        $param->{'unpublished_on_date'} = format_ts( "%Y-%m-%d", $obj->unpublished_on, $app->blog,
            $app->user ? $app->user->preferred_language : undef );
        $param->{'unpublished_on_time'} = format_ts( "%H:%M:%S", $obj->unpublished_on, $app->blog,
            $app->user ? $app->user->preferred_language : undef );
##################### MT6
            }
            $before_status = $obj->status;
            $param->{tags} = join(',',@{$obj->{__tags}});

            my $cat_id;
            my @category_list;
            my %places;

            my $primary = $obj->cache_property( 'category' );
            push @category_list, $primary->id if $primary;

            my $secondary = $obj->cache_property( 'categories' );
            foreach my $c (@{$secondary}) {
                push @category_list, $c->id;
            }

            my $cats = join(',',@category_list);
            if ( defined $cats ) {
                if ( my @cats = grep { $_ =~ /^\d+/ } split /,/, $cats ) {
                    $cat_id = $cats[0];
                    %places = map { $_ => 1 } @cats;
                }
            }
            my $data = $app->_build_category_list(
                blog_id => $q->param('blog_id'),
                markers => 1,
                type    => $model->container_type,
            );
            my $top_cat = $cat_id;
            my @sel_cats;
            my $cat_tree = [];
            if ( $type eq 'page' ) {
                push @$cat_tree,
                    {
                    id       => -1,
                    label    => '/',
                    basename => '/',
                    path     => [],
                    };
                $top_cat ||= -1;
            }
            foreach (@$data) {
                next unless exists $_->{category_id};
                if ( $type eq 'page' ) {
                    $_->{category_path_ids} ||= [];
                    unshift @{ $_->{category_path_ids} }, -1;
                }
                push @$cat_tree,
                    {
                    id       => $_->{category_id},
                    label    => $_->{category_label} . ( $type eq 'page' ? '/' : '' ),
                    basename => $_->{category_basename}
                        . ( $type eq 'page' ? '/' : '' ),
                    path   => $_->{category_path_ids} || [],
                    fields => $_->{category_fields}   || [],
                    };
                push @sel_cats, $_->{category_id}
                    if $places{ $_->{category_id} }
                        && $_->{category_id} != $cat_id;
            }
            $param->{category_tree} = $cat_tree;
            unshift @sel_cats, $top_cat if defined $top_cat && $top_cat ne "";
            $param->{selected_category_loop}   = \@sel_cats;
            $param->{have_multiple_categories} = scalar @$data > 1;

            if ( $rev && @$rev ) {
                if ($q->param('r')) {
                    my @objs2 = $model->load($id);
                    my $obj2 = shift @objs2 || undef;
                    my $rev2 = $obj2->load_revision( { rev_number => $revision->rev_number } );
                    my $obj3 = $rev2->[0] if ( $rev2 && @$rev2 );
                    $param->{status} = $obj3->status;
                    $param->{rev_number} = $q->param('r');
                    if ($revision->rev_number == $q->param('r')) {
                        $param->{loaded_revision} = 0;
                    } else {
                        $param->{loaded_revision} = 1;
                    }
                }
            }

            my @obj_assets = MT::ObjectAsset->load(
                          {
                              object_id => $obj->id
                          } );
            my @asset_ids;
            if (@obj_assets) {
                @asset_ids = map { $_->asset_id } @obj_assets;
            }

            my $assets = [];
            foreach my $asset_id (@asset_ids) {
                my $asset = MT::Asset->load($asset_id);
                next if !$asset;

                my $asset_1;
                if ( $asset->class eq 'image' ) {
                    $asset_1 = {
                        asset_id    => $asset->id,
                        asset_name  => $asset->file_name,
                        asset_thumb => $asset->thumbnail_url( Width => 100 ),
                        asset_type  => $asset->class
                    };
                }
                else {
                    $asset_1 = {
                        asset_id   => $asset->id,
                        asset_name => $asset->file_name,
                        asset_type => $asset->class
                    };
                }
                push @{$assets}, $asset_1;
            }
            $param->{asset_loop} = $assets;

            $param->{'saved_snapshot'} = 1 if $q->param('saved_snapshot');
            $param->{'resend_mail'} = 1 if $q->param('resend_mail');
            $param->{'send_preview'} = 1 if $q->param('send_preview');
        }
    } else {
        my @objs = $model->load($id);
        $obj = shift @objs || undef;

        if ($q->param('r')) {
            my $rev = $obj->load_revision( { rev_number => $q->param('r') } );
            if ( $rev && @$rev ) {
                $obj = $rev->[0];
                $param->{rev_date} = format_ts( "%Y-%m-%d %H:%M:%S",
                    $obj->modified_on, $app->blog,
                    $app->user ? $app->user->preferred_language : undef ) if $obj;
            }
        }
    }

    ### preview bug fix
    } else {
        my $revision = get_lastest_revision($app);
        my $obj = get_lastest_revision_object($app, $revision) if $revision;
        $before_status = $obj->status if $obj;
    }
    ###
    if ( $param->{status} ) {
        if ($param->{status} == 1) {
            $param->{status_draft} = 1;
        }
        if ($param->{status} == 2) {
            $param->{status_publish} = 1;
        }
        if ($param->{status} == 3) {
            $param->{status_review} = 1;
        }
        if ($param->{status} == 10 || $param->{status} == 40) {
            $param->{status_publish_draft} = 1;
            $param->{status_draft} = 0;
            $param->{status_review} = 0;
        }
        if ($param->{status} == 20) {
            $param->{status_publish_review} = 1;
            $param->{status_draft} = 0;
            $param->{status_review} = 0;
        }
        if ($param->{status} > 1000 && $param->{status} < 2000 ) {
            my $order = int( substr( $param->{status}, 1, 3 ) );
            $param->{'status_review' . $order} = 1;
            $param->{status_review} = 1;
        }
        if ($param->{status} > 2000) {
            my $order = int( substr( $param->{status}, 1, 3 ) );
            $param->{'status_publish_review' . $order} = 1;
            $param->{status_publish_review} = 1;
            $param->{status_draft} = 0;
            $param->{status_review} = 0;
        }
        if ( $param->{status} > 1000 ) {
            $param->{current_order} = int( substr( $param->{status}, 1, 3 ) );
        }

        ### preview bug fix
        if ($reviewer && $q->param('reedit')) {
            if ($param->{status} == 10 || $param->{status} == 20) {
                $param->{status_publish} = 1;
            }
        }
        ###
    }

    # エラーありの場合,エラー発生直前の内容を反映させるため更新履歴は利用しない
    if ( $param->{error} && $param->{status} ) {
        $param->{status_draft} = 0;
        $param->{status_review} = 0;
        $param->{status_publish} = 0;
        $param->{status_publish_draft} = 0;
        $param->{status_publish_review} = 0;

        # ライター
        if ( $reviewee ) {

            # 新規作成または未公開の「未公開（原稿）」「未公開（承認待ち）」エラーは「未公開（原稿）」に戻す
            if ($param->{status} == 1 || $param->{status} == 3) {
                $param->{route} = 1;
                $param->{status} = 1;
                $param->{status_draft} = 1;
            }

            # 「公開」での「公開（下書き）」エラーは「公開」に戻す
            # カスタムフィールドについてはsave_snapshot_wfなのでエラーにならない
            if ($param->{status} == 2) {
                $param->{route} = 3;
                $param->{status} = 2;
                $param->{status_publish} = 1;
            }

            # 「公開（下書き）」での「公開（下書き）」「公開（承認待ち）」エラーは「公開（下書き）」に戻す
            if ($param->{status} == 10 || $param->{status} == 20) {
                $param->{route} = 4;
                $param->{status} = 2;
                $param->{status_publish_draft} = 1;
            }

            # 新規作成または未公開の多段階の「未公開（原稿）」「未公開（承認待ち）」エラーは「未公開（原稿）」に戻す
            if ( $param->{status} > 1000 && $param->{status} < 2000 ) {
                $param->{route} = 10;
                $param->{status} = 1;
                $param->{status_review1} = 0;
                $param->{status_draft} = 1;
            }

            # 公開後の多段階の「公開（承認待ち）」エラーは「公開（下書き）」に戻す
            if ( $param->{status} > 2000 ) {
                $param->{route} = 9;
                $param->{status} = 2;
                $param->{status_publish_draft} = 1;
                $param->{status_publish_review1} = 0;
                $param->{status_publish_review} = 0;
            }
        }

        # 承認者
        if ( $reviewer ) {

            # 未公開の「承認」「差し戻し」エラーは「未公開（承認待ち）」に戻す
            if (( $param->{status} == 1 || $param->{status} == 2 ) && $before_status == 3 ) {
                $param->{route} = 5;
                $param->{status_review} = 1;
            }

            # 公開後の「承認」「差し戻し」エラーは「公開（承認待ち）」に戻す
            if (( $param->{status} == 2 || $param->{status} == 40 ) && $before_status == 20 ) {
                $param->{route} = 6;
                $param->{status_publish_review} = 1;
            }

            # 未公開の多段階の「承認依頼」エラーは「公開（承認待ち）」に戻す
            if ( $param->{status} > 1000 && $param->{status} < 2000 ) {
                $param->{route} = 11;
                $param->{status}--;
                $param->{current_order}--;
                my $order = int( substr( $param->{status}, 1, 3 ) );
                $param->{'status_review' . $order} = 1;
                $param->{status_review} = 1;
            }

            # 未公開の多段階の「承認」「差し戻し」エラーは「公開（承認待ち）」に戻す
            if (( $param->{status} == 1 || $param->{status} == 2 ) && ( $before_status > 1000 && $before_status < 2000 )) {
                $param->{route} = 12;
                $param->{status} = $before_status;
                $param->{current_order} = int( substr( $param->{status}, 1, 3 ) );
                my $order = int( substr( $param->{status}, 1, 3 ) );
                $param->{'status_review' . $order} = 1;
                $param->{status_review} = 1;
            }

            # 公開後の多段階の「承認依頼」エラーは「公開（承認待ち）」に戻す
            if ( $param->{status} > 2000 ) {
                $param->{route} = 7;
                $param->{status}--;
                $param->{current_order}--;
                my $order = int( substr( $param->{status}, 1, 3 ) );
                $param->{'status_publish_review' . $order} = 1;
                $param->{status_publish_review} = 1;
            }

            # 公開後の多段階の「承認」「差し戻し」エラーは「公開（承認待ち）」に戻す
            if (( $param->{status} == 40 || $param->{status} == 2 ) && $before_status > 2000 ) {
                $param->{route} = 8;
                $param->{status} = $before_status;
                $param->{current_order} = int( substr( $param->{status}, 1, 3 ) );
                my $order = int( substr( $param->{status}, 1, 3 ) );
                $param->{'status_publish_review' . $order} = 1;
                $param->{status_publish_review} = 1;
            }
        }
    }

    my $obj_class = MT->model($type);
    my $meta_data;

    # システム管理者
    if ($app->user->is_superuser()) {
        $obj = $obj_class->load($id);
        $meta_data = get_meta($obj);

    # ワークフローユーザー
    } elsif ($reviewer || $reviewee) {
        if ($param->{error}) {
            $obj = $obj_class->load($id);
            $meta_data = get_meta($obj);
        } else {

            # ワークフロー対象記事
            if ($param->{entry}) {
                $meta_data = get_meta($param->{entry});

            # ワークフロー対象外の記事を想定
            } else {
                $obj = $obj_class->load($id);
                $meta_data = get_meta($obj);
            }
        }
    # 上記以外のユーザー
    } else {
        $obj = $obj_class->load($id);
        $meta_data = get_meta($obj);
    }

    for my $key (keys %$meta_data) {
        $param->{'field.' . $key} = $meta_data->{$key};
    }
    $param->{__fields_pre_loaded__} = $meta_data ? 1 : 0;

    my $plugin = MT->component('Workflow');
    $param->{rest_day} = $plugin->translate('can use [_1]', rest_day());

    1;
}

sub add_status_for_revision {
    my ($cb, $app, $template) = @_;

    my $old;
    my $new;

    $old = <<HTML;
<mt:var name="pager" value="\$hide_pager" default="0">
HTML
    $old = quotemeta($old);

    $new = <<HTML;
<style type="text/css">
.list-revision-dialog .col.status {
    width: 22%;
}
</style>
<mt:var name="pager" value="\$hide_pager" default="0">
HTML

    $$template =~ s/$old/$new/;


    $old = <<HTML;
      <mt:else name="rev_status" eq="5">
        <span class="icon-left-wide icon-spam"><__trans phrase="Spam"></span>
HTML
    $old = quotemeta($old);

    my $plugin = MT->component('Workflow');
    my $text_status_publish_draft =  $plugin->translate('Published (Draft)');
    my $text_status_publish_review =  $plugin->translate('Published (Review)');
    my $text_status_publish_no_approval =  $plugin->translate('No Approval');
    $new = <<HTML;
      <mt:else name="rev_status" eq="5">
        <span class="icon-left-wide icon-spam"><__trans phrase="Spam"></span>
      <mt:else name="rev_status" eq="10">
        <span class="icon-left-wide2 icon-success-draft"> $text_status_publish_draft</span>
      <mt:else name="rev_status" eq="20">
        <span class="icon-left-wide2 icon-success-review"> $text_status_publish_review</span>
      <mt:else name="rev_status" eq="40">
        <span class="icon-left-wide2 icon-success-draft"> $text_status_publish_no_approval</span>
HTML

    my @approver_list = MT::Approver->load({ blog_id => $app->blog->id });
    my $html;
    my $text_status_review;
    for my $approver ( @approver_list ) {
        my $order = $approver->order;
        my $name = $approver->name;
        my $status_review = sprintf "1%03d", $order;
        my $status_publish_review = sprintf "2%03d", $order;

        $text_status_review = $plugin->translate('Unpublished (Review) of [_1]', $name);
        $text_status_publish_review = $plugin->translate('Published (Review) of [_1]', $name);

        $html .= qq{<mt:else name="rev_status" eq="$status_review"><span class="icon-left-wide icon-warning$order"> $text_status_review</span>};
        $html .= qq{<mt:else name="rev_status" eq="$status_publish_review"><span class="icon-left-wide2 icon-success-review$order"> $text_status_publish_review</span>};
    }
    $new .= $html;

    $$template =~ s/$old/$new/;
}

sub add_css {
    my ($cb, $app, $template) = @_;

    my $old;
    my $new;

    $old = <<HTML;
<mt:var name="html_head">
HTML
    $old = quotemeta($old);

    $new = <<HTML;
<mt:setvarblock name="html_head" append="1">
<style type="text/css">
#entry-status-widget .status {
    position: static!important;
    margin-bottom: 15px;
}
.icon-left-wide2 {
    background-position: left center;
    background-repeat: no-repeat;
    padding-left: 25px;
}
.icon-success-draft {
    background-image: url("<mt:staticwebpath>plugins/Workflow/images/publish_draft.gif");
}
.icon-success-review {
    background-image: url("<mt:staticwebpath>plugins/Workflow/images/publish_review.gif");
}

.listing .status-publish-draft img {
    background-image: url("<mt:staticwebpath>plugins/Workflow/images/publish_draft.gif");
}
.listing .status-publish-review img {
    background-image: url("<mt:staticwebpath>plugins/Workflow/images/publish_review.gif");
}

/* Blog Stats */
.blog-stats-widget .entry-status-publish-draft {
    background-image:url("<mt:staticwebpath>plugins/Workflow/images/publish_draft.gif");
}
.blog-stats-widget .entry-status-publish-review {
    background-image:url("<mt:staticwebpath>plugins/Workflow/images/publish_review.gif");
}
.blog-stats-widget .content.entry-status-publish-draft,
.blog-stats-widget .content.entry-status-publish-review {
    padding-left: 33px;
}
HTML

    my $html;
    my $max_order = get_max_order( $app );
    for ( my $i = 1; $i <= $max_order; $i++ ) {
        $html .= <<HTML;
.icon-success-review$i {
    background-image: url("<mt:staticwebpath>plugins/Workflow/images/publish_review.gif");
}
.icon-warning$i {
    background-image: url("<mt:staticwebpath>plugins/Workflow/images/review.gif");
}
HTML
    }
    $new .= $html;

    $new .= <<HTML;
</style>
</mt:setvarblock>
<mt:var name="html_head">
HTML

    $$template =~ s/$old/$new/;
}

sub add_status_for_list {
    my $cb = shift;
    my ( $app, $res, $objs ) = @_;

    my $type = 'entry';

    for my $data (@{$res->{objects}}) {
        my $model = $app->model($type);
        my $rev_model = $app->model($type.':revision');
        return 1 unless $rev_model;

        my ($terms, $args);
        $terms->{'entry_id'} = $data->[0];
        $args->{sort} = 'rev_number';
        $args->{direction} = 'descend';
        $args->{limit} = 1;
        my @revs = $rev_model->load($terms, $args);
        my $revision = shift @revs || undef;

        my $entry;
        if ($revision) {
            my @objs = $model->load($data->[0]);
            my $obj = shift @objs || undef;
            my $rev = $obj->load_revision( { rev_number => $revision->rev_number } );
            if ( $rev && @$rev ) {
                $entry = $rev->[0];
            }
        }
        if ($entry) {
            for (my $i=1; $i<=2; $i++) {
                if ($entry->status == 10) {
                    $data->[$i] =~ s!/images/status_icons/"!/plugins/Workflow/images/publish_draft.gif"!;
                    $data->[$i] =~ s!/images/status_icons/success\.gif!/plugins/Workflow/images/publish_draft.gif!;
                } elsif ($entry->status == 40) {
                    $data->[$i] =~ s!/images/status_icons/"!/plugins/Workflow/images/publish_draft.gif"!;
                    $data->[$i] =~ s!/images/status_icons/!/plugins/Workflow/images/publish_draft.gif!;
                } elsif ($entry->status == 20) {
                    $data->[$i] =~ s!/images/status_icons/"!/plugins/Workflow/images/publish_review.gif"!;
                    $data->[$i] =~ s!/images/status_icons/success\.gif!/plugins/Workflow/images/publish_review.gif!;
                } elsif ($entry->status > 2000) {
                    my $order = int( substr( $entry->status, 1, 3 ) );
                    $data->[$i] =~ s!/images/status_icons/"!/plugins/Workflow/images/publish_review.gif"!;
                    $data->[$i] =~ s!/images/status_icons/success\.gif!/plugins/Workflow/images/publish_review.gif!;
                } elsif ($entry->status > 1000) {
                    my $order = int( substr( $entry->status, 1, 3 ) );
                    $data->[$i] =~ s!/images/status_icons/"!/plugins/Workflow/images/review.gif"!;
                    $data->[$i] =~ s!/images/status_icons/success\.gif!/plugins/Workflow/images/review.gif!;
                }
            }
        }
    }
}

sub add_status {
    my ($cb, $app, $template) = @_;

    my $old;
    my $new;

if ( $app->user->is_superuser ) {
    $old = <<HTML;
    <mt:if name="status_review">
      <option value="3" selected="selected"><__trans phrase="Unpublished (Review)"></option>
    <mt:else name="status_spam">
HTML
    $old = quotemeta($old);

    $new = <<HTML;
    <mt:if name="status_spam">
HTML
    $$template =~ s/$old/$new/;
}

    return if !use_revision($app);
    my $reviewee = $app->can_do('request_entry_approval');
    my $reviewer = $app->can_do('approve_entry');

    my $plugin = MT->component('Workflow');
    my $publish_status_draft = $plugin->translate('Published (Draft)');
    my $publish_status_review = $plugin->translate('Published (Review)');

    my $author_name = $app->user->name;
    my $approver = MT::Approver->load( { author => { like => "'%${approver_prefix}${author_name}${approver_prefix}%'" }, blog_id => $app->blog->id }, {});
    my $approver_name = $approver->name if $approver || '';
    my $my_order = $approver->order if $approver || 0;
    my $next_order = $my_order + 1;
    my $max_order = get_max_order( $app );

##################### MT6
    $old = <<HTML;
  <mt:unless name="new_object">
    <mt:if name="status_draft">
    <span class="icon-left-wide icon-draft"><__trans phrase="Unpublished (Draft)"></span>
    <mt:else name="status_review">
    <span class="icon-left-wide icon-warning"><__trans phrase="Unpublished (Review)"></span>
    <mt:else name="status_future">
    <span class="icon-left-wide icon-future"><__trans phrase="Scheduled"></span>
    <mt:else name="status_publish">
    <span class="icon-left-wide icon-success"><__trans phrase="Published"></span>
    <mt:else name="status_spam">
    <span class="icon-left-wide icon-spam"><__trans phrase="Unpublished (Spam)"></span>
    <mt:else name="status_unpublish">
    <span class="icon-left-wide icon-unpublish"><__trans phrase="Unpublished (End)"></span>
    </mt:if>
  </mt:unless>
HTML
##################### MT6
    $old = quotemeta($old);

    $new = <<HTML;
  <mt:unless name="new_object">
    <mt:if name="status_draft">
    <span class="icon-left-wide icon-draft"><__trans phrase="Unpublished (Draft)"></span>
HTML

    my @approver_list = MT::Approver->load({ blog_id => $app->blog->id });
    my $html;
    my $text_status_review;
    my $text_status_publish_review;
    for my $approver ( @approver_list ) {
        my $order = $approver->order;
        my $name = $approver->name;
        my $status_review = 'status_review' . $order;
        my $status_publish_review = 'status_publish_review' . $order;

        $text_status_review = $plugin->translate('Unpublished (Review) of [_1]', $name);
        $text_status_publish_review = $plugin->translate('Published (Review) of [_1]', $name);

        $html .= qq{<mt:else name="$status_review"><span class="icon-left-wide icon-warning$order">$text_status_review</span>};
        $html .= qq{<mt:else name="$status_publish_review"><span class="icon-left-wide2 icon-success-review$order">$text_status_publish_review</span>};
    }
    $new .= $html;

    $new .= <<HTML;
    <mt:else name="status_review">
    <span class="icon-left-wide icon-warning"><__trans phrase="Unpublished (Review)"></span>
    <mt:else name="status_future">
    <span class="icon-left-wide icon-future"><__trans phrase="Scheduled"></span>
    <mt:else name="status_publish_draft">
    <span class="icon-left-wide2 icon-success-draft">$publish_status_draft</span>
    <mt:else name="status_publish_review">
    <span class="icon-left-wide2 icon-success-review">$publish_status_review</span>
    <mt:else name="status_publish">
    <span class="icon-left-wide icon-success"><__trans phrase="Published"></span>
    <mt:else name="status_spam">
    <span class="icon-left-wide icon-spam"><__trans phrase="Unpublished (Spam)"></span>
    <mt:else name="status_unpublish">
    <span class="icon-left-wide icon-unpublish"><__trans phrase="Unpublished (End)"></span>
    </mt:if>
  </mt:unless>
HTML

    $$template =~ s/$old/$new/;

    $old = <<HERE;
  <div id="quickpost">
    <p><mt:var name="quickpost_js"></p>
  </div>
HERE
    $old = quotemeta($old);
    $new = <<HERE;
  <div id="quickpost">
    <p><mt:var name="quickpost_js"></p>
  </div>
<div style="margin-top:20px">
<mt:var name="rest_day">
</div>
HERE
        $$template =~ s!$old!$new!;

#    return if !can_workflow($app);

    $old = <<HERE;
    <mt:setvar name="publish_button_text" value="<__trans phrase="Publish">">
    <mt:setvarblock name="publish_button_title"><__trans phrase="Publish this [_1]" params="<mt:var name="object_label">" escape="html"></mt:setvarblock>
HERE
    $old = quotemeta($old);
    $new = <<HERE;
    <mt:setvar name="publish_button_text" value="<__trans phrase="Publish">">
    <mt:setvarblock name="publish_button_title"><__trans phrase="Publish this [_1]" params="<mt:var name="object_label">" escape="html"></mt:setvarblock>
    <mt:setvar name="review_button_text" value="<__trans phrase="Review">">
    <mt:setvarblock name="review_button_title"><__trans phrase="Review this [_1]" params="<mt:var name="object_label">" escape="html"></mt:setvarblock>
    <mt:setvar name="update_button_text" value="<__trans phrase="Update">">
    <mt:setvarblock name="update_button_title"><__trans phrase="Update this [_1]" params="<mt:var name="object_label">" escape="html"></mt:setvarblock>
HERE
##################### MT6
    $$template =~ s!$old!$new!g;
##################### MT6
    $old = <<HTML;
</mt:if>
<mt:if name="loaded_revision">
HTML
    $old = quotemeta($old);

    $new = <<HTML;
</mt:if>
<mt:if name="loaded_revision">
  <mt:if name="status_publish_draft">
    <mt:var name="current_status_text" value="success-draft icon-left-wide2">
    <mt:setvarblock name="current_status_label" value="">$publish_status_draft</mt:setvarblock>
  <mt:else name="status_publish_review">
    <mt:var name="current_status_text" value="success-review icon-left-wide2">
    <mt:setvarblock name="current_status_label" value="">$publish_status_review</mt:setvarblock>
  </mt:if>
HTML
# no change
#    $$template =~ s/$old/$new/;

    $old = <<HTML;
<mt:setvarblock name="related_content">
HTML
    $old = quotemeta($old);

    $new = <<HTML;
<mt:setvarblock name="related_content" append="1">
<mtapp:widget
   id="entry-status-widget"
   class="status-widget"
   label="<__trans phrase="Status">">
  <mt:if name="new_object">
  <mt:else>
  <div class="revision-info">
      <mt:if name="rev_date">
    <p><__trans phrase="Revision: <strong>[_1]</strong>" params="<mt:var name="rev_date" escape="html">"></p>
    <a href="<mt:var name="script_url">?__mode=list_revision2&_type=<mt:var name="object_type" default="link">&id=<mt:var name="id" escape="url">&blog_id=<mt:var name="blog_id">&r=<mt:var name="rev_number" escape="html">" class="mt-open-dialog" title="<__trans phrase="View revisions of this [_1]" params="<mt:var name="object_label">">"><__trans phrase="View revisions"></a>
      <mt:else>
    <p class="zero-state"><__trans phrase="No revision(s) associated with this [_1]" params="<mt:var name="object_label" lower_case="1" escape="html">"></p>
      </mt:if>
  </div>
  <ul>
    <li><__trans phrase="[_1] - Created by [_2]" params="<mt:date ts="\$created_on_ts" relative="1" _default="\$created_on_formatted" format="%b %e %Y">%%<mt:var name="created_by" escape="html" escape="html">"></li>
      <mt:if name="status_publish">
    <li><__trans phrase="[_1] - Published by [_2]" params="<MTDate ts="\$authored_on_ts" relative="1" _default="\$authored_on_formatted" format="%b %e %Y">%%<mt:var name="author_name" escape="html" escape="html">"></li>
      </mt:if>
      <mt:if name="modified_by">
    <li><__trans phrase="[_1] - Edited by [_2]" params="<MTDate ts="\$modified_on_ts" relative="1" _default="\$modified_on_formatted" format="%b %e %Y">%%<mt:var name="modified_by" escape="html" escape="html">"></li>
      </mt:if>
  </ul>
  </mt:if>
</mtapp:widget>
</mt:setvarblock>
<mt:setvarblock name="related_content">
HTML
    $$template =~ s/$old/$new/;

    $old = <<HTML;
<div id="msg-block">
HTML
    $old = quotemeta($old);

    my $resend_mail = $plugin->translate('Resend Mail.');
    my $send_preview = $plugin->translate('Send Preview.');
    my $publish_draft = $plugin->translate('Publish Draft.');
    my $publish_review = $plugin->translate('Publish Review.');

    $new = <<HTML;
<div id="msg-block">
<mt:if name="resend_mail">
  <mtapp:statusmsg
     id="resend-mail"
     class="success">
     $resend_mail
  </mtapp:statusmsg>
</mt:if>
<mt:if name="send_preview">
  <mtapp:statusmsg
     id="send-preview"
     class="success">
     $send_preview
  </mtapp:statusmsg>
</mt:if>
<mt:if name="saved_snapshot">
<mt:if name="status_publish_draft">
  <mtapp:statusmsg
     id="resend-mail"
     class="success">
     $publish_draft
  </mtapp:statusmsg>
<mt:else name="status_publish_review">
  <mtapp:statusmsg
     id="resend-mail"
     class="success">
     $publish_review
  </mtapp:statusmsg>
</mt:if>
</mt:if>

HTML
    $$template =~ s/$old/$new/;

    $old = <<HERE;
<div class="delete-action">
  <a
     href="javascript:void(0)"
     mt:command="do-remove-items"
     mt:object-singular="<mt:var name="object_label" lower_case="1" escape="html">"
     mt:object-plural="<mt:var name="object_label_plural" lower_case="1" escape="html">"
     mt:object-type="<mt:var name="object_type" escape="html">" mt:blog-id="<mt:var name="blog_id">"
     type="submit"
     title="<mt:var name="delete_button_title">"
     class="delete"
     accesskey="x">
    <__trans phrase="Delete">
  </a>
</div>
HERE

    $old = quotemeta($old);
    $$template =~ s!$old!!;

    if ($reviewee) {
##################### MT6
        $old = <<HERE;
    <select name="status" id="status" class="full" onchange="highlightSwitch(this)">
      <option value="1"<mt:if name="status_draft"> selected="selected"</mt:if>><__trans phrase="Unpublished (Draft)"></option>
      <option value="2"<mt:if name="status_publish"> selected="selected"</mt:if>><__trans phrase="Published"></option>
    <mt:if name="status_review">
      <option value="3" selected="selected"><__trans phrase="Unpublished (Review)"></option>
    <mt:else name="status_spam">
      <option value="5" selected="selected"><__trans phrase="Unpublished (Spam)"></option>
    </mt:if>
    <mt:if name="new_object">
      <option value="4"<mt:if name="status_future"> selected="selected"</mt:if>><__trans phrase="Scheduled"></option>
    <mt:else>
      <mt:unless name="status_publish">
      <option value="4"<mt:if name="status_future"> selected="selected"</mt:if>><__trans phrase="Scheduled"></option>
      </mt:unless>
    </mt:if>
    <mt:if name="status_unpublish">
      <option value="6" selected="selected"><__trans phrase="Unpublished (End)"></option>
    </mt:if>
    </select>
HERE
##################### MT6
    $old = quotemeta($old);
    $new = <<HERE;
<mt:SetVar name="hidden_button" value="0" />
<mt:if name="status_review">
  <mt:SetVar name="hidden_button" value="1" />
</mt:if>
<mt:if name="status_publish_review">
  <mt:SetVar name="hidden_button" value="1" />
</mt:if>
<__trans_section component="Workflow">

    <select<mt:if name="hidden_button"> disabled="disabled"</mt:if> name="status" id="status" class="full" onchange="highlightSwitch(this)">
    <mt:if name="status_draft">
      <option value="1"<mt:if name="status_draft"> selected="selected"</mt:if>><__trans phrase="Unpublished (Draft)"></option>
      <option value="1001"><__trans phrase="Unpublished (Review)"></option>
    <mt:else name="status_review">
      <option value="3"><__trans phrase="Unpublished (Review)"></option>
    <mt:else name="status_publish_draft">
      <option value="10" selected="selected"><__trans phrase="Published (Draft)"></option>
      <option value="2001"><__trans phrase="Request Approve after Publish"></option>
    <mt:else name="status_publish_review">
      <option value="20"><__trans phrase="Published (Review)"></option>
    <mt:else name="status_publish">
      <mt:if name="new_object">
        <option value="1"<mt:if name="status_draft"> selected="selected"</mt:if>><__trans phrase="Unpublished (Draft)"></option>
        <option value="1001"<mt:if name="status_review"> selected="selected"</mt:if>><__trans phrase="Unpublished (Review)"></option>
      <mt:else>
        <option value="10" selected="selected"><__trans phrase="Published (Draft)"></option>
        <option value="2001"><__trans phrase="Request Approve after Publish"></option>
      </mt:if>
    </mt:if>
    <mt:if name="status_unpublish">
      <option value="1"<mt:if name="status_draft"> selected="selected"</mt:if>><__trans phrase="Unpublished (Draft)"></option>
      <option value="1001"><__trans phrase="Unpublished (Review)"></option>
    </mt:if>
    </select>
</__trans_section>
HERE
    $$template =~ s!$old!$new!;

    $old = <<HERE;
    <div class="save-action">
      <button
         name="status"
         type="submit"
         title="<mt:var name="button_title">"
         class="publish action primary button">
        <mt:var name="button_text">
      </button>
    </div>
HERE

    $old = quotemeta($old);
    $new = <<HERE;
<mt:unless name="hidden_button">
    <div class="save-action">
      <button
         name="status"
         type="submit"
         title="<mt:var name="button_title">"
         class="publish action primary button">
        <mt:var name="button_text">
      </button>
    </div>
</mt:unless>
HERE
    $$template =~ s!$old!$new!;

    $old = <<HERE;
<mt:if name="use_revision">
  <mtapp:setting
     id="revision-note"
     label="<__trans phrase="Change note">"
     label_class="top-label">
    <input type="text" name="revision-note" id="revision-note" class="text full" value="<mt:var name="revision-note" escape="html">" />
  </mtapp:setting>
</mt:if>
HERE

    $old = quotemeta($old);
    $new = <<HERE;
<mt:unless name="hidden_button">
<mt:if name="use_revision">
  <mtapp:setting
     id="revision-note"
     label="<__trans phrase="Change note">"
     label_class="top-label">
    <input type="text" name="revision-note" id="revision-note" class="text full" value="<mt:var name="revision-note" escape="html">" />
  </mtapp:setting>
</mt:if>
</mt:unless>
HERE
    $$template =~ s!$old!$new!;

    $old = <<HERE;
    function setSaveButtonText () {
        var selectedStatus = jQuery('select#status').val();
        if (selectedStatus == 2) {
            jQuery('button.publish').text('<mt:var name="publish_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="publish_button_title">');
        } else if (selectedStatus == 4) {
            jQuery('button.publish').text('<mt:var name="scheduled_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="scheduled_button_title">');
        } else {
            jQuery('button.publish').text('<mt:var name="draft_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="draft_button_title">');
        }
    }
HERE

my $review_button_text = $plugin->translate('Request Approve');
my $publish_review_button_text = $plugin->translate('Request Approve after Publish');

    $old = quotemeta($old);
    $new = <<HERE;
    function setSaveButtonText () {
        var selectedStatus = jQuery('select#status').val();
        if (selectedStatus == 3) {
            jQuery('#workflow-field').css('display','block');
            jQuery('button.publish').text('<mt:var name="draft_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="draft_button_title">');
            jQuery('.preview-action').hide();
        } else if (selectedStatus == 4) {
            jQuery('button.publish').text('<mt:var name="scheduled_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="scheduled_button_title">');
        } else if (selectedStatus == 10) {
            jQuery('#workflow-field').css('display','none');
            jQuery('button.publish').text('<mt:var name="draft_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="draft_button_title">');
            jQuery('#workflow-field').css('display','none');
            jQuery('.preview-action').show();
        } else if (selectedStatus == 20) {
            jQuery('#workflow-field').css('display','block');
            jQuery('button.publish').text('<mt:var name="draft_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="draft_button_title">');
            jQuery('.preview-action').hide();
        } else if (selectedStatus == 1) {
            jQuery('#workflow-field').css('display','none');
            jQuery('button.publish').text('<mt:var name="draft_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="draft_button_title">');
            jQuery('.preview-action').show();
        } else {
            jQuery('button.publish').text('<mt:var name="draft_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="draft_button_title">');
        }
    }

    jQuery('#status').on('change', function() {
        var changedStatus = jQuery('#status option:selected').val();
        if (changedStatus == 10 || changedStatus == 1 || changedStatus == 2 ) {
            jQuery('#workflow-field').css('display','none');
        } else {
            jQuery('#workflow-field').css('display','block');
        }
    });
HERE
        $$template =~ s!$old!$new!;

    $old = <<HERE;
    jQuery('button.publish').click(function(event) {
        app.clearDirty();
        app.stopAutoSave();
        jQuery('form#entry_form input[name=__mode]').val('save_entry');
        jQuery('form#entry_form').removeAttr('target');
        jQuery(this).attr('disabled', 'disabled');
        jQuery('form#entry_form').submit();
    });
  </mt:if>
HERE

    $old = quotemeta($old);
    $new = <<HERE;
    jQuery('button.publish').click(function(event) {
        app.clearDirty();
        app.stopAutoSave();
        jQuery('form#entry_form input[name=__mode]').val('save_entry');
        jQuery('form#entry_form').removeAttr('target');
        jQuery(this).attr('disabled', 'disabled');
        var selectedStatus = jQuery('select#status').val();
        if (selectedStatus == 10 || selectedStatus == 20 || selectedStatus == 40 || selectedStatus > 2000) {
            jQuery('form#entry_form input[name=__mode]').val('save_snapshot_wf');
        }
//        jQuery('form#entry_form').submit();
    });
  <mt:else>
    jQuery('button.publish').click(function(event) {
        var selectedStatus = jQuery('select#status').val();
        if (selectedStatus == 10 || selectedStatus == 20 || selectedStatus == 40 || selectedStatus > 2000) {
            jQuery('form#entry_form input[name=__mode]').val('save_snapshot_wf');
        }
//        jQuery('form#entry_form').submit();
    });
  </mt:if>
HERE
        $$template =~ s!$old!$new!;

    # for 6.1
    $old = <<HERE;
  <mt:if name="config.previewinnewwindow">
    jQuery('button.publish').click(function(event) {
        jQuery('form#entry_form input[name=__mode]').val('save_entry');
        jQuery('form#entry_form').removeAttr('target');
        jQuery('form#entry_form').attr('mt:once', '1');
    });
  </mt:if>
HERE

    $old = quotemeta($old);
    $new = <<HERE;
  <mt:if name="config.previewinnewwindow">
    jQuery('button.publish').click(function(event) {
        jQuery('form#entry_form input[name=__mode]').val('save_entry');
        jQuery('form#entry_form').removeAttr('target');
        jQuery('form#entry_form').attr('mt:once', '1');
        var selectedStatus = jQuery('select#status').val();
        if (selectedStatus == 10 || selectedStatus == 20 || selectedStatus == 40 || selectedStatus > 2000) {
            jQuery('form#entry_form input[name=__mode]').val('save_snapshot_wf');
        }
    });
  <mt:else>
    jQuery('button.publish').click(function(event) {
        var selectedStatus = jQuery('select#status').val();
        if (selectedStatus == 10 || selectedStatus == 20 || selectedStatus == 40 || selectedStatus > 2000) {
            jQuery('form#entry_form input[name=__mode]').val('save_snapshot_wf');
        }
    });
  </mt:if>
HERE
        $$template =~ s!$old!$new!;

    } else {
##################### MT6
        $old = <<HERE;
    <select name="status" id="status" class="full" onchange="highlightSwitch(this)">
      <option value="1"<mt:if name="status_draft"> selected="selected"</mt:if>><__trans phrase="Unpublished (Draft)"></option>
      <option value="2"<mt:if name="status_publish"> selected="selected"</mt:if>><__trans phrase="Published"></option>
    <mt:if name="status_review">
      <option value="3" selected="selected"><__trans phrase="Unpublished (Review)"></option>
    <mt:else name="status_spam">
      <option value="5" selected="selected"><__trans phrase="Unpublished (Spam)"></option>
    </mt:if>
    <mt:if name="new_object">
      <option value="4"<mt:if name="status_future"> selected="selected"</mt:if>><__trans phrase="Scheduled"></option>
    <mt:else>
      <mt:unless name="status_publish">
      <option value="4"<mt:if name="status_future"> selected="selected"</mt:if>><__trans phrase="Scheduled"></option>
      </mt:unless>
    </mt:if>
    <mt:if name="status_unpublish">
      <option value="6" selected="selected"><__trans phrase="Unpublished (End)"></option>
    </mt:if>
    </select>
HERE
##################### MT6
    $old = quotemeta($old);

    my $status_review = 2;
    my $status_publish_review = 2;
    $status_review = sprintf "1%03d", $next_order if $my_order;
    $status_publish_review = sprintf "2%03d", $next_order if $my_order;
    $status_publish_review = $status_review = 2 if $max_order == $my_order;

    $new = <<HERE;
<mt:SetVar name="display_button" value="0" />
<mt:if name="status_review">
  <mt:SetVar name="display_button" value="1" />
</mt:if>
<mt:if name="status_publish_review">
  <mt:SetVar name="display_button" value="1" />
</mt:if>

<mt:if name="current_order" ne="$my_order">
  <mt:SetVar name="display_button" value="0" />
</mt:if>

<__trans_section component="Workflow">
    <input type="hidden" name="approver_name" value="$approver_name" />

    <select<mt:unless name="display_button"> disabled="disabled"</mt:unless> name="status" id="status" class="full" onchange="highlightSwitch(this)">
    <mt:if name="status_review">

<mt:if name="current_order" gt="$my_order">
  <option value="1" selected="selected"><__trans phrase="Done approval"></option>
</mt:if>
<mt:if name="current_order" lt="$my_order">
  <option value="1" selected="selected"><__trans phrase="Doing approval"></option>
</mt:if>
      <option value="1"<mt:if name="status_draft"> selected="selected"</mt:if>><__trans phrase="No Approval"></option>
      <option value="$status_review"<mt:if name="status_publish"> selected="selected"</mt:if>><__trans phrase="Approve"></option>
    <mt:else name="status_draft">
      <option value="1"><__trans phrase="Unpublished (Draft)"></option>
    <mt:else name="status_publish_draft">
      <option value="10" selected="selected"><__trans phrase="Published (Draft)"></option>
    <mt:else name="status_publish_review">
    <mt:if name="current_order" gt="$my_order">
      <option value="40" selected="selected"><__trans phrase="Done approval"></option>
    </mt:if>
    <mt:if name="current_order" lt="$my_order">
      <option value="40" selected="selected"><__trans phrase="Doing approval"></option>
    </mt:if>
      <option value="40" <mt:if name="current_order" eq="$my_order">selected="selected"</mt:if>><__trans phrase="No Approval"></option>
      <option value="$status_publish_review"><__trans phrase="Approve"></option>
    <mt:else name="status_publish">
      <option value="2"><__trans phrase="Approve"></option>
    </mt:if>
    <mt:if name="status_unpublish">
      <option value="6" selected="selected"><__trans phrase="Unpublished (End)"></option>
    </mt:if>
    </select>
</__trans_section>
HERE

    $$template =~ s!$old!$new!;

    $old = <<HERE;
    <div class="save-action">
      <button
         name="status"
         type="submit"
         title="<mt:var name="button_title">"
         class="publish action primary button">
        <mt:var name="button_text">
      </button>
    </div>
HERE

    $old = quotemeta($old);
    $new = <<HERE;
<mt:if name="display_button">
    <div class="save-action">
      <button
         name="status"
         type="submit"
         title="<mt:var name="button_title">"
         class="publish action primary button">
        <mt:var name="button_text">
      </button>
    </div>
</mt:if>
HERE
    $$template =~ s!$old!$new!;

    $old = <<HERE;
    function setSaveButtonText () {
        var selectedStatus = jQuery('select#status').val();
        if (selectedStatus == 2) {
            jQuery('button.publish').text('<mt:var name="publish_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="publish_button_title">');
        } else if (selectedStatus == 4) {
            jQuery('button.publish').text('<mt:var name="scheduled_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="scheduled_button_title">');
        } else {
            jQuery('button.publish').text('<mt:var name="draft_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="draft_button_title">');
        }
    }
HERE

    $old = quotemeta($old);

    $new = <<HERE;
    function setSaveButtonText () {
        var selectedStatus = jQuery('select#status').val();
        if (selectedStatus == 2) {
            jQuery('button.publish').text('<mt:var name="publish_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="publish_button_title">');
        } else if (selectedStatus == 3) {
            jQuery('button.publish').text('<mt:var name="review_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="review_button_title">');
        } else if (selectedStatus == 4) {
            jQuery('button.publish').text('<mt:var name="scheduled_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="scheduled_button_title">');
        } else if (selectedStatus == 30) {
            jQuery('button.publish').text('<mt:var name="button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="button_title">');
        } else if (selectedStatus == 40) {
            jQuery('button.publish').text('<mt:var name="draft_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="draft_button_title">');
        } else {
            jQuery('button.publish').text('<mt:var name="draft_button_text">');
            jQuery('button.publish').attr('title', '<mt:var name="draft_button_title">');
        }
    }

    function change_message_field (changedStatus) {
<mt:var name="max_order" value="$max_order">
<mt:if name="max_order" ne="$my_order">
        if (changedStatus == 10 || changedStatus == 40 || changedStatus == 1 || changedStatus == 2 ) {
            jQuery('#workflow-request-approve-field').css('display','none');
            jQuery('#workflow-no-approve-message-field').css('display','block');
            jQuery('#no_approve_message').prop("disabled", false);
            jQuery('#approve_message').prop("disabled", true);
        } else {
            jQuery('#workflow-request-approve-field').css('display','block');
            jQuery('#workflow-no-approve-message-field').css('display','none');
            jQuery('#no_approve_message').prop("disabled", true);
            jQuery('#approve_message').prop("disabled", false);
        }
<mt:else>
        if (changedStatus == 1 || changedStatus == 40) {
            jQuery('#workflow-approve-message-field').css('display','none');
            jQuery('#workflow-no-approve-message-field').css('display','block');
            jQuery('#no_approve_message').prop("disabled", false);
            jQuery('#approve_message').prop("disabled", true);
        } else {
            jQuery('#workflow-approve-message-field').css('display','block');
            jQuery('#workflow-no-approve-message-field').css('display','none');
            jQuery('#no_approve_message').prop("disabled", true);
            jQuery('#approve_message').prop("disabled", false);
        }
</mt:if>
    }

    // ステータス切り替え時のメッセージエリア制御
    jQuery('#status').on('change', function() {
        var changedStatus = jQuery('#status option:selected').val();
        change_message_field(changedStatus);
    });

    // ページ読み込み時のメッセージエリア制御
    var changedStatus = jQuery('#status option:selected').val();
    change_message_field(changedStatus);
HERE

    $$template =~ s!$old!$new!;

    $old = <<HERE;
    jQuery('button.publish').click(function(event) {
        app.clearDirty();
        app.stopAutoSave();
        jQuery('form#entry_form input[name=__mode]').val('save_entry');
        jQuery('form#entry_form').removeAttr('target');
        jQuery(this).attr('disabled', 'disabled');
        jQuery('form#entry_form').submit();
    });
  </mt:if>
HERE

    $old = quotemeta($old);
    $new = <<HERE;
    jQuery('button.publish').click(function(event) {
        app.clearDirty();
        app.stopAutoSave();
        jQuery('form#entry_form input[name=__mode]').val('save_entry');
        jQuery('form#entry_form').removeAttr('target');
        jQuery(this).attr('disabled', 'disabled');
        var selectedStatus = jQuery('select#status').val();
        if (selectedStatus == 10 || selectedStatus == 20 || selectedStatus == 40 || selectedStatus > 2000) {
            jQuery('form#entry_form input[name=__mode]').val('save_snapshot_wf');
        }
//        jQuery('form#entry_form').submit();
    });
  <mt:else>
    jQuery('button.publish').click(function(event) {
        var selectedStatus = jQuery('select#status').val();
        if (selectedStatus == 10 || selectedStatus == 20 || selectedStatus == 40 || selectedStatus > 2000) {
            jQuery('form#entry_form input[name=__mode]').val('save_snapshot_wf');
        }
//        jQuery('form#entry_form').submit();
    });
  </mt:if>
HERE
        $$template =~ s!$old!$new!;

    # for 6.1
    $old = <<HERE;
  <mt:if name="config.previewinnewwindow">
    jQuery('button.publish').click(function(event) {
        jQuery('form#entry_form input[name=__mode]').val('save_entry');
        jQuery('form#entry_form').removeAttr('target');
        jQuery('form#entry_form').attr('mt:once', '1');
    });
  </mt:if>
HERE

    $old = quotemeta($old);
    $new = <<HERE;
  <mt:if name="config.previewinnewwindow">
    jQuery('button.publish').click(function(event) {
        jQuery('form#entry_form input[name=__mode]').val('save_entry');
        jQuery('form#entry_form').removeAttr('target');
        jQuery('form#entry_form').attr('mt:once', '1');
        var selectedStatus = jQuery('select#status').val();
        if (selectedStatus == 10 || selectedStatus == 20 || selectedStatus == 40 || selectedStatus > 2000) {
            jQuery('form#entry_form input[name=__mode]').val('save_snapshot_wf');
        }
    });
  <mt:else>
    jQuery('button.publish').click(function(event) {
        var selectedStatus = jQuery('select#status').val();
        if (selectedStatus == 10 || selectedStatus == 20 || selectedStatus == 40 || selectedStatus > 2000) {
            jQuery('form#entry_form input[name=__mode]').val('save_snapshot_wf');
        }
    });
  </mt:if>
HERE
        $$template =~ s!$old!$new!;

    }

    $old = <<HERE;
<div id="category-field"<mt:unless name="disp_prefs_show_category"> class="hidden"</mt:unless>>
HERE
    $old = quotemeta($old);

    if ($reviewee) {

        my @authors = get_authors_with_order($app, $plugin, 1);
        my $address_list;

        my $approver = MT::Approver->load({ order => 1, blog_id => $app->blog->id },{});
        my $send_approve_mail = $approver->send_approve_mail;

        if ( $send_approve_mail == 1 ) {

            foreach my $author (@authors) {
                $address_list .= '<li><input type="checkbox" name="address" id="address'.$author->{id}.'" value="author'.$author->{id}.'" checked="checked" class="cb" /> <label for="address'.$author->{id}.'">'.$author->{name}.'</label></li>';
            }

        } elsif ( $send_approve_mail == 3 ) {

            foreach my $author (@authors) {
                $address_list .= '<li><input type="checkbox" name="address" id="address'.$author->{id}.'" value="author'.$author->{id}.'" class="cb" /> <label for="address'.$author->{id}.'">'.$author->{name}.'</label></li>';
            }

        } elsif ( $send_approve_mail == 2 ) {

            foreach my $author (@authors) {
                $address_list .= '<li><input type="checkbox" name="address" id="address" value="author'.$author->{id}.'" checked="checked" class="cb" disabled="disabled" /> <label for="address">'.$author->{name}.'</label></li>';
                $address_list .= '<input type="hidden" name="address" id="address" value="author'.$author->{id}.'" />';
            }

        } else {
            $address_list = '';
        }

        if ($address_list) {
            $address_list = '<ul>'.$address_list.'</ul>';
        } else {
            $address_list = $plugin->translate('No Address');
        }

        my $h2_message_request_approve = $plugin->translate('Request Approve');
        my $h2_message_resend_mail = $plugin->translate('Resend Mail');
        my $body_address_list_message = $plugin->translate('Address for Approval');
        my $mail_message = $plugin->translate('Approval note');
        my $resend_mail = $plugin->translate('Send Mail');

        my $body_start = <<HTML;
<mtapp:setting
    id="allow-feedback"
    label="$body_address_list_message"
    label_class="top-label"
    help_page="entries"
    help_section="workflow_mail">
    $address_list
</mtapp:setting>
<mtapp:setting
    id="approval-note"
    label="$mail_message"
    label_class="top-label">
HTML

        my $body_end = <<HTML;
</mtapp:setting>
HTML

        $new = <<HERE;
<mt:if name="status_review">
  <mt:var name="review" value="1" />
</mt:if>
<mt:if name="status_publish_review">
  <mt:var name="review" value="1" />
</mt:if>
<mt:unless name="review">
    <div id="workflow-field" style="display: none;">
        <mtapp:widget
            id="workflow-widget"
            label="$h2_message_request_approve">
            $body_start
    <textarea name="request_approve_message" class="text full low" id="request_approve_message" cols="" rows=""><mt:var name="message" escape="html"></textarea>
            $body_end
        </mtapp:widget>
    </div>
<mt:else>
    <mt:unless name="error">

    <mt:if name="current_order" eq="1">
        <div id="workflow-field" style="display: none;">
            <mtapp:widget
                id="workflow-widget"
                label="$h2_message_resend_mail">
                $body_start
    <textarea name="resend_message" class="text full low" id="resend_message" cols="" rows=""><mt:var name="message" escape="html"></textarea>
                $body_end
                <button
                    mt:mode="resend_mail"
                    type="submit"
                    title="<__trans phrase="Send mail">"
                    class="save draft action button"
                    value="1"
                    >$resend_mail</button>
            </mtapp:widget>
        </div>
    </mt:if>

    </mt:unless>
</mt:unless>

<div id="category-field"<mt:unless name="disp_prefs_show_category"> class="hidden"</mt:unless>>
HERE

    } elsif ($reviewer) {

        my $approver = MT::Approver->load( { author => { like => "'%${approver_prefix}${author_name}${approver_prefix}%'" }, blog_id => $app->blog->id }, {} );
        my $next_approve_order = ($approver->order) + 1;
        my $max_order = get_max_order( $app );

        my @authors = get_authors_with_order($app, $plugin, $next_approve_order) if $next_approve_order < $max_order + 1;
        my $address_list;

        my $next_approver = MT::Approver->load({ order => $next_order, blog_id => $app->blog->id },{});

if ( $next_approver ) {
        my $send_approve_mail = $next_approver->send_approve_mail;

        if ( $send_approve_mail == 1 ) {

            foreach my $author (@authors) {
                $address_list .= '<li><input type="checkbox" name="address" id="address'.$author->{id}.'" value="author'.$author->{id}.'" checked="checked" class="cb" /> <label for="address'.$author->{id}.'">'.$author->{name}.'</label></li>';
            }

        } elsif ( $send_approve_mail == 3 ) {

            foreach my $author (@authors) {
                $address_list .= '<li><input type="checkbox" name="address" id="address'.$author->{id}.'" value="author'.$author->{id}.'" class="cb" /> <label for="address'.$author->{id}.'">'.$author->{name}.'</label></li>';
            }

        } elsif ( $send_approve_mail == 2 ) {

            foreach my $author (@authors) {
                $address_list .= '<li><input type="checkbox" name="address" id="address" value="author'.$author->{id}.'" checked="checked" class="cb" disabled="disabled" /> <label for="address">'.$author->{name}.'</label></li>';
                $address_list .= '<input type="hidden" name="address" id="address" value="author'.$author->{id}.'" /></li>';
            }

        } else {
            $address_list = '';
        }

        if ($address_list) {
            $address_list = '<ul>'.$address_list.'</ul>';
        } else {
            $address_list = $plugin->translate('No Address');
        }
}

        my $h2_message_request_approve = $plugin->translate('Request Approve');
        my $h2_message_resend_mail = $plugin->translate('Resend Mail');
        my $body_address_list_message = $plugin->translate('Address for Next approval');
        my $mail_message = $plugin->translate('Request approval message to next approver');
        my $resend_mail = $plugin->translate('Send Mail');

        my $body_start = <<HTML;
<mtapp:setting
    id="allow-feedback"
    label="$body_address_list_message"
    label_class="top-label"
    help_page="entries"
    help_section="workflow_mail">
$address_list
</mtapp:setting>
<mtapp:setting
    id="approval-note"
    label="$mail_message"
    label_class="top-label">
HTML

        my $body_end = <<HTML;
</mtapp:setting>
HTML

        $new = <<HERE;
<mt:if name="status_review">
  <mt:var name="review" value="0" />
</mt:if>
<mt:if name="status_publish_review">
  <mt:var name="review" value="0" />
</mt:if>
<mt:unless name="review">
    <mt:var name="max_order" value="$max_order">
    <mt:if name="max_order" ne="$my_order">
        <div id="workflow-request-approve-field" style="display: none;">
            <mtapp:widget
                id="workflow-widget"
                label="$h2_message_request_approve">
                $body_start
    <textarea name="request_approve_message" class="text full low" id="request_approve_message" cols="" rows=""><mt:var name="message" escape="html"></textarea>
                $body_end
            </mtapp:widget>
        </div>
    </mt:if>
</mt:unless>
HERE

        my $h2_approve_message = $plugin->translate('Approve');
        my $h2_no_approve_message = $plugin->translate('No Approval');
        my $h2_preview_message = $plugin->translate('Request Preview');
        my $h2_preview_message_review = $plugin->translate('Send Mail');

        my $approve_mail_message = $plugin->translate('Approval message to writer');
        my $no_approve_mail_message = $plugin->translate('No Approval message to writer');
        my $preview_message = $plugin->translate('Address for Preview');
        my $preview_mail_message = $plugin->translate('Approval note');
        my $send_mail = $plugin->translate('Preview Mail');
        my $else_email = $plugin->translate('Else Email');

        my $iter = MT::AddressBook->load_iter( { blog_id => [ $app->blog->id, 0 ] });
        my $address_list;

        while (my $address = $iter->()) {
            $address_list .= '<li><input type="checkbox" name="address" id="address'.$address->id.'" value="address'.$address->id.'" checked="checked" class="cb" /> <label for="address'.$address->id.'">'.$address->name.'</label></li>';
        }
        if ($address_list) {
            $address_list = '<ul>'.$address_list.'</ul>';
        } else {
            $address_list = $plugin->translate('No Address');
        }

        $new .= <<HERE;
<mt:if name="status_review">
  <mt:var name="review" value="1" />
</mt:if>
<mt:if name="status_publish_review">
  <mt:var name="review" value="1" />
</mt:if>
<mt:if name="review">

<mt:if name="current_order" eq="$my_order">
    <mt:var name="max_order" value="$max_order">
    <mt:if name="max_order" eq="$my_order">
        <div id="workflow-approve-message-field" style="display: none;">
            <mtapp:widget
                id="workflow-widget"
                label="$h2_approve_message">
                <mtapp:setting
                    id="approval-note"
                    label="$approve_mail_message"
                    label_class="top-label">
                    <textarea name="message" class="text full low" cols="" rows="" id="approve_message"><mt:var name="message" escape="html"></textarea>
                </mtapp:setting>
            </mtapp:widget>
        </div>
    </mt:if>

    <div id="workflow-no-approve-message-field" style="display: none;">
        <mtapp:widget
            id="workflow-widget"
            label="$h2_no_approve_message">
            <mtapp:setting
                id="approval-note"
                label="$no_approve_mail_message"
                label_class="top-label">
                <textarea name="message" class="text full low" cols="" rows="" id="no_approve_message"><mt:var name="message" escape="html"></textarea>
            </mtapp:setting>
        </mtapp:widget>
    </div>
</mt:if>

<mt:if name="current_order" op="--" eq="$my_order">
    <div id="workflow-field">
        <mtapp:widget
            id="workflow-widget"
            label="$h2_message_resend_mail">
            $body_start
    <textarea name="resend_message" class="text full low" id="resend_message" cols="" rows=""><mt:var name="message" escape="html"></textarea>
            $body_end
            <button
                mt:mode="resend_mail"
                type="submit"
                title="<__trans phrase="Send mail">"
                class="save draft action button"
                value="1"
                >$resend_mail</button>
        </mtapp:widget>
    </div>
</mt:if>

<div id="workflow-preview-field">
    <mtapp:widget
        id="workflow-widget"
        label="$h2_preview_message">

        <mtapp:setting
            id="preview-messag"
            label="$preview_message"
            label_class="top-label"
            help_page="entries"
            help_section="workflow_mail">
$address_list
        </mtapp:setting>
        <mtapp:setting
            id="else-email"
            label="$else_email"
            label_class="top-label"
            help_page="entries"
            help_section="workflow_mail">
<input type="text" id="else_email" name="else_email" class="text full" />
        </mtapp:setting>
        <mtapp:setting
            id="approval-note"
            label="$preview_mail_message"
            label_class="top-label">
            <textarea name="preview_message" class="text full low" cols="" rows="" id="preview_message"><mt:var name="preview_message" escape="html"></textarea>
        </mtapp:setting>
        <button
            mt:mode="request_preview_mail"
            type="submit"
            title="<__trans phrase="Send mail">"
            class="save draft action button"
            value="1"
            >$send_mail</button>
    </mtapp:widget>
</div>
</mt:if>
<div id="category-field"<mt:unless name="disp_prefs_show_category"> class="hidden"</mt:unless>>
HERE
    }

    $$template =~ s!$old!$new!;

    my $workflow_display_delete_button = $plugin->get_config_value('workflow_display_delete_button', 'blog:' . $app->blog->id);

    if ($reviewee && !$workflow_display_delete_button) {
    $old = <<HERE;
  <mt:if name="id">
  <div class="delete-action">
    <a
       href="javascript:void(0)"
       mt:command="do-remove-items"
       mt:object-singular="<mt:var name="object_label" lower_case="1" escape="html">"
       mt:object-plural="<mt:var name="object_label_plural" lower_case="1" escape="html">"
       mt:object-type="<mt:var name="object_type" escape="html">" mt:blog-id="<mt:var name="blog_id">"
       type="submit"
       title="<mt:var name="delete_button_title">"
       class="delete"
       accesskey="x">
      <__trans phrase="Delete">
    </a>
  </div>
  </mt:if>
HERE
    $old = quotemeta($old);

    $$template =~ s!$old!!;
    }
}

sub itemset_action_widget {
    my ($cb, $app, $template) = @_;

    return if !can_workflow($app);

    my $q = $app->{query};
    my $blog_id = $q->param('blog_id');

    my $plugin = MT->component('Workflow');
    my $workflow_display_unpublish_option = $plugin->get_config_value('workflow_display_unpublish_option', 'blog:' . $blog_id);
    my $reviewee = $app->can_do('request_entry_approval');

    if ($reviewee && !$workflow_display_unpublish_option) {

    my $old = <<HERE;
<select name="plugin_action_selector" onchange="updatePluginAction(this)">
    <option value="0"><__trans phrase="More actions..."></option>
    <mt:loop name="list_actions">
    <option value="<mt:var name="key">"><mt:var name="label"></option>
    </mt:loop>
    <mt:loop name="more_list_actions">
        <mt:if name="__first__">
    <optgroup label="<__trans phrase='Plugin Actions'>">
        </mt:if>
        <option value="<mt:var name="key">"><mt:var name="label"></option>
        <mt:if name="__last__">
    </optgroup>
        </mt:if>
    </mt:loop>
</select>
<mt:if name="form_id">
<button
     class="small-button mt-<mt:var name="form_id">-action"
     ><__trans phrase="Go"></button>
<mt:else>
<button
     class="small-button mt-<mt:var name="object_type">-listing-form-action"
     ><__trans phrase="Go"></button>
</mt:if>
HERE
    $old = quotemeta($old);

    $$template =~ s!$old!!;

    }
}

sub _get_authors {
    my ($app, $plugin) = @_;

    require MT::Association;
    require MT::Role;
    my $blog = $app->blog;

    my $role_name = $plugin->translate('Approver');
    my $role = MT::Role->load( { name => $role_name } );
    my $role_id = $role->id;

    my $iter = MT::Author->load_iter(undef, {
                              join => MT::Association->join_on('author_id', {
                                  role_id => $role_id,
                                  blog_id => $blog->id
                              }, { unique => 1 })
                          });

    my @authors;
    while (my $author = $iter->()) {
        push(@authors, { id => $author->id, name => $author->name });
    }
    return @authors;
}

1;
